package hhenson.core;

/**
 * Represents that an object manages field operations.
 * In number theory a field provides ring and group homomorphisms
 * with the property that the homomorphisms have inverse functions.
 * That is the homomophisms are bi-jections that are one to one and onto.
 * Or something to that effect.
 * All this says is that a class implementing this interface will provide
 * basic add, subtract, multiply, and divide operations.
 * Operations should only take place with values within the same domain
 * values, i.e. in the domain of Money, only Money should be passed as
 * a value. We may implement methods to allow other domain elements,
 * but then we would apply a mapping function to allow the element
 * to be part of this domain.
 * 
 * @author Howard Henson
 */
public interface Field {
	/**
	 * Provides addition function.
	 * This returns a reference to self so that mathamatical operations
	 * can be chained. Just keep track of order of work, since this may
	 * be significant.
	 */
	public Field add(Field field) throws FieldMismatchException;
	
	/**
	 * Provides addition function that returns the result as new object.
	 * This should not modify the properties of the object, but
	 * should return a new object with the addition performed on it.
	 */
	public Field addNew(Field field) throws FieldMismatchException;
	
	/**
	 * Provides subtraction function.
	 * This returns a reference to self so that mathamatical operations
	 * can be chained. Just keep track of order of work, since this may
	 * be significant.
	 */
	public Field subtract(Field field) throws FieldMismatchException;
	
	/**
	 * Provides subtraction function that returns the result as new object.
	 * This should not modify the properties of the object, but
	 * should return a new object with the subtraction performed on it.
	 */
	public Field subtractNew(Field field) throws FieldMismatchException;
	
	/**
	 * Provides multiply function.
	 * This returns a reference to self so that mathamatical operations
	 * can be chained. Just keep track of order of work, since this may
	 * be significant.
	 */
	public Field multiply(Field field) throws FieldMismatchException;
	
	/**
	 * Provides multiplication function that returns the result as new object.
	 * This should not modify the properties of the object, but
	 * should return a new object with the multiplication performed on it.
	 */
	public Field multiplyNew(Field field) throws FieldMismatchException;
	
	/**
	 * Provides divide function.
	 * This returns a reference to self so that mathamatical operations
	 * can be chained. Just keep track of order of work, since this may
	 * be significant.
	 */
	public Field divide( Field field) throws FieldMismatchException;
	
	/**
	 * Provides divide function that returns the result as new object.
	 * This should not modify the properties of the object, but
	 * should return a new object with the division performed on it.
	 */
	public Field divideNew( Field field) throws FieldMismatchException;
	
}
