package hhenson.oodb.core;

import goodsjpi.Persistent;
import hhenson.core.*;




/**
* Represents a bounded ID object. This is used to identify if an object should be
* treated as an upper or lower bound. This is to simplify the construction of
* complex search id's when using an ID to find a collection of information.
* 
* The important methods are the isUpperBound and isLowerBound which denote which
* bound this search ID would refer to.
* 
* This could be persisted, though I wouldn't imediately see any specific reason
* to do so.
* @author Howard Henson
* @version 1.0
* @generator Enterprise Architect
* @created 26-Aug-2002 09:09:02
*/

public class IDBound extends Persistent implements Comparable {

	/**
	 * The result to return from compareTo when a greater than b
	 */
	public static final int GREATER_THAN = 1;
	/**
	 * The result to return from a.compareTo(b) when a less than b
	 */
	public static final int LESS_THAN = -1;
	
	/**
	* If true then this is an upper bound, otherwise it is a lower bound.
	*/
	private boolean upperBound;

	/**
	* The id that this object refers to
	*/
	private ID id;
	
	protected IDBound(){

	}

	public ID getId(){
		return id;
	}

	/**
	* Constructs a new IDBound with the relavant ID and bound.
	* @param    newID    The ID object in question.
	* @param    isUpperBound    True if this is an upper bound, false if it is a lower bound
	*/
	public IDBound(ID newID, boolean isUpperBound){
		id = newID;
		upperBound = isUpperBound;
	}

	/**
	* Denotes this object represents an upper bound.
	*/
	public boolean isUpperBound(){
		return upperBound;
	}

	/**
	* Denotes that this object represents a lower bound.
	*/
	public boolean isLowerBound(){
		return !upperBound;
	}


	/**
	 * @see java.lang.Comparable#compareTo(Object)
	 */
	public int compareTo(Object obj) {
		if( obj instanceof IDBound ){
			IDBound idBound = (IDBound) obj;
			if( upperBound ){
				if( idBound.upperBound ){
					return id.compareTo(idBound.id);
				} else {
					return GREATER_THAN;
				}
			} else {
				if( idBound.upperBound ){
					return LESS_THAN;
				} else {
					return id.compareTo(idBound.id);
				}
			}
		}
				
		return -((Comparable)obj).compareTo(this);
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return id.toString();
	}

}