package hhenson.oodb.core;

import goodsjpi.*;
import goodslib.*;
import hhenson.core.*;

import java.util.*;

/**
 * Implements a structured OodbIDImpl.
 * A structured OodbIDImpl is based on a dot seperated list of strings
 * to construct the id. The OodbIDImpl provides basic search functionality.
 * 
 * @author Howard Henson
 */
public abstract class StructuredID extends OodbIDImpl implements Matchable {

	private final static byte RECURSIVE_WILDCARD = '~';
	private final static byte WILDCARD = '*';

	/**
	 * The event name itself.
	 */
	private char[] sIDData;
	private transient String tNameElement;

	/**
	 * The offsets to each element within the string.
	 */
	private transient int[] tOffSets;
	private int hashCode = 0;
	private boolean hasWildCard = false;
	private boolean hasRecursiveWildCard = false;
	private boolean looseMatch = false;

	protected String getNameElement() {
		if (tNameElement == null) {
			if (sIDData != null) {
				tNameElement = String.valueOf(sIDData, 1, (int) sIDData[0]);
			}
		}
		return tNameElement;
	}

	protected void setNameElement(String str) {
		Metaobject.modify();
		tNameElement = str;
		tOffSets = extractTokens(str);
		//		System.out.println( "Offsets: " + tOffSets.length );
		sIDData = new char[1 + str.length() + tOffSets.length];
		sIDData[0] = (char) str.length();
		//		System.out.println( "String: " + str.length() + " arr: " + sIDData.length );
		System.arraycopy(str.toCharArray(), 0, sIDData, 1, str.length());
		for (int i = 0; i < tOffSets.length; i++) {
			char ch = (char) tOffSets[i];
			sIDData[str.length() + i + 1] = ch;
		}
	}

	protected void setRaw(char[] raw) {
		Metaobject.modify();
		sIDData = new char[raw.length];
		System.arraycopy(raw, 0, sIDData, 0, raw.length);
	}

	protected int[] getOffSets() {
		if (tOffSets == null) {
			if (sIDData != null) {
				tOffSets = new int[sIDData.length - 1 - sIDData[0]];
				for (int i = 0; i < tOffSets.length; i++) {
					tOffSets[i] = (int) sIDData[(int) sIDData[0] + i + 1];
					if(tOffSets[i] == (int)((char)((int)-1))){
						tOffSets[i] = -1;
					}
				}
			}
		}
		return tOffSets;
	}

	public void setLooseMatch(boolean isLooseMatchable) {
		looseMatch = isLooseMatchable;
	}

	public boolean isLooseMatch() {
		return looseMatch;
	}

	public boolean hasWildCard() {
		return hasWildCard;
	}

	public boolean hasRecursiveWildCard() {
		return hasRecursiveWildCard;
	}

	public StructuredID() {
		super();
	}

	/**
	* Parses the name into its component elements.
	* @param    eventName
	*/
	protected void setStructuredID(String nameElement) {
		setNameElement(nameElement);
		hashCode = nameElement.hashCode();
		if (nameElement.charAt(nameElement.length() - 1)
			== (char) RECURSIVE_WILDCARD) {
			hasRecursiveWildCard = true;
		} else {
			hasRecursiveWildCard = false;
		}
		if (nameElement.indexOf(WILDCARD) != -1) {
			hasWildCard = true;
		} else {
			hasWildCard = false;
		}
	}

	public boolean isMatch(Object obj) {
		if (obj instanceof StructuredID) {
			return isMatch((StructuredID) obj);
		}
		return false;
	}

	public boolean isExpandable() {
		return (hasWildCard() || hasRecursiveWildCard());
	}

	/**
	 * Determines if the event name is a match for this event name. This takes wild carding into account when matching.
	 * @param    eventName
	 */
	public boolean isMatch(StructuredID structuredID) {
		if (getOffSets().length != structuredID.getOffSets().length) {
			//If the event name lengths aren't equal and there are no recursive wild cars,
			//then we will immediately return a false isMatch...
			boolean looseMatch = this.looseMatch || structuredID.looseMatch;
			if (!(hasRecursiveWildCard() || (looseMatch && hasWildCard))
				&& !(structuredID.hasRecursiveWildCard()
					|| (looseMatch && structuredID.hasWildCard))) {
				return false;
			}
			if (getOffSets().length < structuredID.getOffSets().length) {
				if (hasRecursiveWildCard()
					|| (looseMatch
						&& (hasWildCard || structuredID.hasWildCard))) {
					//compare to nameElements.length-3
					//(Reason length-3 = count for array index plus last index of wildcard.
					return checkMatch(
						structuredID,
						(hasRecursiveWildCard)
							? getOffSets().length - 3
							: getOffSets().length - 2,
						hasWildCard()
							|| structuredID.hasWildCard()
							|| hasRecursiveWildCard);
				} else {
					return false;
				}
			} else {
				if (structuredID.hasRecursiveWildCard()
					|| (looseMatch
						&& (hasWildCard || structuredID.hasWildCard))) {
					//compare to nameElements.length-3
					//(Reason same as above...)
					return checkMatch(
						structuredID,
						(structuredID.hasRecursiveWildCard)
							? structuredID.getOffSets().length - 3
							: structuredID.getOffSets().length - 2,
						hasWildCard()
							|| structuredID.hasWildCard()
							|| structuredID.hasRecursiveWildCard);
				} else {
					return false;
				}
			}
		}
		//If we get here then the event name lengths are equal...
		//Must test for recursive wild cards before returning default opertion.
		if (hasRecursiveWildCard() || structuredID.hasRecursiveWildCard()) {
			return checkMatch(
				structuredID,
				getOffSets().length - 3,
				hasWildCard() || structuredID.hasWildCard());
		}
		//Default return operation...
		//compare to nameElements.length-1
		//(Reason length-1 = count for array only)
		return checkMatch(
			structuredID,
			getOffSets().length - 2,
			hasWildCard() || structuredID.hasWildCard());
	}

	/**
	 * checks the elements to depth in the array.
	 */
	private boolean checkMatch(
		StructuredID structuredID,
		int depth,
		boolean doWildCardCheck) {
		for (int elementNumber = 0; elementNumber <= depth; elementNumber++) {
			int diff1 =
				getOffSets()[elementNumber
					+ 1]
					- (getOffSets()[elementNumber] + 1);
			int diff2 =
				structuredID.getOffSets()[elementNumber
					+ 1]
					- (structuredID.getOffSets()[elementNumber] + 1);
			int beginIndex1 = getOffSets()[elementNumber] + 1;
			int beginIndex2 = structuredID.getOffSets()[elementNumber] + 1;
			if (diff1 != diff2) {
				if (!doWildCardCheck) {
					return false;
				}
				if (diff1 > 1 && diff2 > 1) {
					return false;
				}
				if (getNameElement().charAt(beginIndex1) != WILDCARD
					&& structuredID.getNameElement().charAt(beginIndex2)
						!= WILDCARD) {
					return false;
				}
				if (looseMatch) {
					return true;
				}
				//We match since one of the elements must be a wildcard
			} else if (
				!getNameElement().regionMatches(
					beginIndex1,
					structuredID.getNameElement(),
					beginIndex2,
					diff1)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Compares two event names.
	 * This comparison will be exact comparisons, not expanding wildcards etc.
	 * @param    obj
	 */
	public int compareTo(Object obj) {
		if (obj instanceof StructuredID) {
			StructuredID structuredID = (StructuredID) obj;
			if (isExpandable() || structuredID.isExpandable()) {
				return compareWithWildCards(structuredID);
			}
			return getNameElement().compareTo(structuredID.getNameElement());
		}

		if (obj instanceof IDBound) {
			if (isMatch(((IDBound)obj).getId())) {
				return (((IDBound) obj).isUpperBound())
					? IDBound.LESS_THAN
					: IDBound.GREATER_THAN;
			}
			compareTo(((IDBound) obj).getId());
		}

		if (obj instanceof String) {
			return getNameElement().compareTo(obj);
		}
		return getClass().getName().compareTo(obj.getClass().getName());
	}

	/**
	 * Takes wild cards into account when performing comparison.
	 */
	protected int compareWithWildCards(StructuredID structuredID) {
		int compResult = 0;
		int minSize =
			Math.min(getOffSets().length, structuredID.getOffSets().length);
		int rVal = 0;
		if( getOffSets().length > minSize ){
			rVal = IDBound.GREATER_THAN;
		}
		if( structuredID.getOffSets().length > minSize ){
			rVal = IDBound.LESS_THAN;
		}
		for (int c = 0; c < minSize - 1; c++) {
			int begin1 = getOffSets()[c] + 1;
			int begin2 = structuredID.getOffSets()[c] + 1;
			int end1 = getOffSets()[c + 1];
			int end2 = structuredID.getOffSets()[c + 1];
			String str1 = null;
			String str2 = null;
			try {
				str1 = getNameElement().substring(begin1, end1);
				str2 = structuredID.getNameElement().substring(begin2, end2);
			} catch (Exception e) {
				System.err.println(
					"Str: "
						+ getNameElement()
						+ " ["
						+ begin1
						+ ", "
						+ end1
						+ "]");
				System.err.println(
					"Str: "
						+ structuredID.getNameElement()
						+ " ["
						+ begin2
						+ ", "
						+ end2
						+ "]");
			}
			if (end1 - begin1 == 1) {
				if (str1.charAt(0) == RECURSIVE_WILDCARD) {
					return 0;
				}
				if (str1.charAt(0) == WILDCARD) {
					continue;
				}
			}
			if (end2 - begin2 == 1) {
				if (str2.charAt(0) == RECURSIVE_WILDCARD) {
					return 0;
				}
				if (str2.charAt(0) == WILDCARD) {
					continue;
				}
			}
			compResult = str1.compareTo(str2);
			if (compResult != 0) {
				return compResult;
			}
		}
		return rVal;
	}

	/**
	 * @param    obj
	 */
	public boolean equals(Object obj) {
		if (obj instanceof String) {
			return this.toString().equals(obj);
		}
		if (obj instanceof StructuredID) {
			return getNameElement().equals(
				((StructuredID) obj).getNameElement());
		}
		return false;
	}

	public int hashCode() {
		return hashCode;
	}

	/**
	 * @param    eventNameString
	 */
	public StructuredID(String eventNameString) {
		setStructuredID(eventNameString);
	}

	public String toString() {
		return getNameElement();
	}

	/**
	 * returns the number of tokens in the name.
	 */
	public int getNumberOfElements() {
		if (getOffSets() == null) {
			return 0;
		}
		return getOffSets().length - 1;
	}

	/**
	 * returns the String element at position n
	 */
	public String getElement(int n) {
		if (n >= getOffSets().length - 1) {
			return null;
		}
		int begin1 = getOffSets()[n] + 1;
		int end1 = getOffSets()[n + 1];
		return getNameElement().substring(begin1, end1);
	}

	/**
	 * extracts the tokens from an event Name.
	 * This extracts all strings that reside within a dot.
	 * That is if the eventName is t1.t2.t3.t4 then the tokens list returned
	 * looks like { t1, t2, t3, t4 }
	 * (posOfDot, ...)
	 * @param eventName the name to process
	 * @returns String[] the string array of strings
	 */
	protected int[] extractTokens(String eventName) {
		ArrayOfInt arrInt = new ArrayOfInt(0, 16);
		arrInt.push(-1);
		for (int c = 0; c < eventName.length(); c++) {
			if (eventName.charAt(c) == '.') {
				arrInt.push(c);
			}
		}
		arrInt.push(eventName.length());
		return arrInt.toArray();
	}

	/**
	 * @see java.lang.Object#clone()
	 */
	public Object clone() {
		StructuredID id;
		try {
			id = (StructuredID) super.clone();
		} catch (Exception e) {
			return null;
		}
		id.hashCode = hashCode;
		id.hasRecursiveWildCard = hasRecursiveWildCard;
		id.hasWildCard = hasWildCard;
		id.looseMatch = looseMatch;
		id.setRaw(this.sIDData);
		return id;
	}

}
