package hhenson.oodb.entity;

import goodsjpi.*;
import hhenson.configuration.*;

/**
 * An address object.
 * Represents a generic implementation of an address. An
 * address is not a standalone object and as such does not
 * contain an ID for unique referencing.
 * Should this object need to be deemed as 'unique' then
 * either the whole object should be part of the key or
 * an external reference key should be associated with the
 * object.
 * @author Howard Henson
 */
public class Address extends Persistent implements Comparable, Configurable {
	private String line1;
	private String line2;
	private String province;
	private String areaCode;
	private int countryCode;

	/**
	 * Constructor for Address.
	 */
	public Address() {
		super();
	}

	/**
	 * Constructor for Address.
	 * @param metaobject
	 */
	public Address(Metaobject metaobject) {
		super(metaobject);
	}

	/**
	 * Returns the areaCode.
	 * @return String
	 */
	public String getAreaCode() {
		return areaCode;
	}

	/**
	 * Returns the countryCode.
	 * @return int
	 */
	public int getCountryCode() {
		return countryCode;
	}

	/**
	 * Returns the line1.
	 * @return String
	 */
	public String getLine1() {
		return line1;
	}

	/**
	 * Returns the line2.
	 * @return String
	 */
	public String getLine2() {
		return line2;
	}

	/**
	 * Returns the province.
	 * @return String
	 */
	public String getProvince() {
		return province;
	}

	/**
	 * Sets the areaCode.
	 * @param areaCode The areaCode to set
	 */
	public void setAreaCode(String areaCode) {
		this.areaCode = areaCode;
	}

	/**
	 * Sets the countryCode.
	 * @param countryCode The countryCode to set
	 */
	public void setCountryCode(int countryCode) {
		this.countryCode = countryCode;
	}

	/**
	 * Sets the line1.
	 * @param line1 The line1 to set
	 */
	public void setLine1(String line1) {
		this.line1 = line1;
	}

	/**
	 * Sets the line2.
	 * @param line2 The line2 to set
	 */
	public void setLine2(String line2) {
		this.line2 = line2;
	}

	/**
	 * Sets the province.
	 * @param province The province to set
	 */
	public void setProvince(String province) {
		this.province = province;
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object obj) {
		if (obj instanceof Address) {
			Address address = (Address) obj;
			if (countryCode != address.getCountryCode()) {
				return false;
			}
			if (!areaCode.equals(address.getAreaCode())) {
				return false;
			}
			if (!province.equals(address.getProvince())) {
				return false;
			}
			if (!line2.equals(address.getLine2())) {
				return false;
			}
			if (!line1.equals(address.getLine1())) {
				return false;
			}
			return true;
		}
		//if it reaches this point it was not of type Address and therefore the answer must be false
		return false;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuffer strBuffer = new StringBuffer();
		if (line1 != null) {
			strBuffer.append(line1);
		}
		if (line2 != null) {
			strBuffer.append(line2);
		}
		if (province != null) {
			strBuffer.append(province);
		}
		if (areaCode != null) {
			strBuffer.append(areaCode);
		}
		strBuffer.append(
			CountryCodes
				.COUNTRY_NAMES[countryCode][CountryCodes
				.LONG_COUNTRY_NAME]);

		return strBuffer.toString();

	}

	/**
	 * @see java.lang.Comparable#compareTo(Object)
	 * The comparable method will be determined in the following manner:
	 * whether the address objects are the same and if not then by 
	 * countryCode; areaCode; Province; line2; line1 (in this order)
	 * If there
	 */
	public int compareTo(Object o) {
		if (o instanceof Address) {
			Address address = (Address) o;
			if (countryCode > address.getCountryCode()) {
				return -1;
			}
			if (countryCode < address.getCountryCode()) {
				return 1;
			}
			if (areaCode.compareTo(address.getAreaCode())!=0) {
				return areaCode.compareTo(address.getAreaCode());
			}
			if (province.compareTo(address.getProvince())!=0) {
				return province.compareTo(address.getProvince());
			}
			if (line2.compareTo(address.getLine2())!=0) {
				return line2.compareTo(address.getLine2());
			}
			if (line1.compareTo(address.getLine1())!=0) {
				return line1.compareTo(address.getLine1());
			}
		}
		System.out.println(
			"Could not compare object, it is not of type Address");
		return this.getClass().getName().compareTo(o.getClass().getName());
	}

	public static final String CFG_LINE1 = "Line1";
	public static final String CFG_LINE2 = "Line2";
	public static final String CFG_AREA_CODE = "AreaCode";
	public static final String CFG_PROVINCE = "Province";
	public static final String CFG_COUNTRY_CODE = "CountryCode";
	/**
	 * @see hhenson.configuration.Configurable#cfgLoad(Configuration)
	 */
	public void cfgLoad(Configuration config) {
		// <Line1>100 Main Str</Line1>
		// <Line2>Suburb</Line2>
		// <AreaCode>2196</AreaCode>
		// <Province>Gauteng</Province>
		// <CountryCode>27</CountryCode>
		line1=(String)config.getValue(CFG_LINE1);
		line2=(String)config.getValue(CFG_LINE2);
		areaCode=(String)config.getValue(CFG_AREA_CODE);
		province=(String)config.getValue(CFG_PROVINCE);
		String valueStr = (String) config.getValue(CFG_COUNTRY_CODE);
		countryCode=CountryCodes.ZA;                 //defaulting it to SA
		if (valueStr != null) {
			countryCode = Integer.parseInt(valueStr); 
		}
	}

	/**
	 * @see hhenson.configuration.Configurable#cfgStore()
	 */
	public Configuration cfgStore() {
		Configuration conf = Configuration.newInstance();
		conf.setValue(CFG_LINE1, line1);
		conf.setValue(CFG_LINE2, line2);
		conf.setValue(CFG_AREA_CODE, areaCode);
		conf.setValue(CFG_PROVINCE, province);
		conf.setValue(CFG_COUNTRY_CODE, Integer.toString(countryCode));
		return conf;
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return line1.hashCode()
			^ line2.hashCode()
			^ province.hashCode()
			^ areaCode.hashCode()
			^ countryCode;
	}

}
