package hhenson.oodb.entity;

import goodsjpi.*;
import hhenson.configuration.*;

/**
 * Holds basic banking details.
 * The details should be verified as they are added to the object.
 * The object does not contain any identifier other than a full property
 * check.
 * @author Howard Henson
 */
public class BankingDetails
	extends Persistent
	implements Comparable, Configurable {

	public static final int CHECK_ACCOUNT = 1;
	public static final int CURRENT_ACCOUNT = 2;
	public static final int TRANSMISSION_ACCOUNT = 3;
	public static final int SAVINGS_ACCOUNT = 4;

	/**
	 * The bank.
	 * This may be a bank code, not sure yet.
	 */
	private String bank;

	/**
	 * The branch code.
	 * This should be checked against a list of valid
	 * branch codes for the asociated bank.
	 */
	private String branchCode;

	/**
	 * The account number.
	 * The check algorithm should be obtained and tested when this is set.
	 */
	private String accountNumber;

	/**
	 * The account type.
	 * This is one of CurrentAccount, CheckAccount, etc.
	 */
	private int accountType;

	/**
	 * Constructor for BankingDetails.
	 */
	public BankingDetails() {
		super();
	}

	/**
	 * Constructor for BankingDetails.
	 */
	public BankingDetails(
		String bank,
		String branchCode,
		String accountNumber,
		int accountType) {
		super();
		setBank(bank);
		setAccountType(accountType);
		setBranchCode(branchCode);
		setAccountNumber(accountNumber);
	}

	/**
	 * Returns the accountNumber.
	 * @return String
	 */
	public String getAccountNumber() {
		return accountNumber;
	}

	/**
	 * Returns the accountType.
	 * @return int
	 */
	public int getAccountType() {
		return accountType;
	}

	/**
	 * Returns the bank.
	 * @return String
	 */
	public String getBank() {
		return bank;
	}

	/**
	 * Returns the brachCode.
	 * @return String
	 */
	public String getBranchCode() {
		return branchCode;
	}

	/**
	 * Sets the accountNumber.
	 * @param accountNumber The accountNumber to set
	 */
	public void setAccountNumber(String accountNumber) {
		this.accountNumber = accountNumber;
	}

	/**
	 * Sets the accountType.
	 * @param accountType The accountType to set
	 */
	public void setAccountType(int accountType) {
		this.accountType = accountType;
	}

	/**
	 * Sets the bank.
	 * @param bank The bank to set
	 */
	public void setBank(String bank) {
		this.bank = bank;
	}

	/**
	 * Sets the brachCode.
	 * @param brachCode The brachCode to set
	 */
	public void setBranchCode(String branchCode) {
		this.branchCode = branchCode;
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object obj) {
		if (obj instanceof BankingDetails) {
			BankingDetails bankingDetails = (BankingDetails) obj;
			if (!bank.equals(bankingDetails.getBank())) {
				return false;
			}
			if (!branchCode.equals(bankingDetails.getBranchCode())) {
				return false;
			}
			if (accountType != bankingDetails.getAccountType()) {
				return false;
			}
			if (!accountNumber.equals(bankingDetails.getAccountNumber())) {
				return false;
			}
			return true;
		}
		//if it reaches this point it was not of type Banking Details and therefore the answer must be false
		return false;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuffer strBuffer = new StringBuffer();
		strBuffer.append(accountNumber);
		//there should be some way of mapping this to a string that has meaning/or is this irrelevant
		strBuffer.append(accountType);
		strBuffer.append(bank);
		strBuffer.append(branchCode);
		return strBuffer.toString();
	}

	/**
	 * @see java.lang.Comparable#compareTo(Object)
	 */
	public int compareTo(Object obj) {
		if (obj instanceof BankingDetails) {
			BankingDetails bankingDetails = (BankingDetails) obj;
			if (bank.compareTo(bankingDetails.getBank()) != 0) {
				return bank.compareTo(bankingDetails.getBank());
			}
			if (branchCode.compareTo(bankingDetails.getBranchCode()) != 0) {
				return branchCode.compareTo(bankingDetails.getBranchCode());
			}
			if (accountType > bankingDetails.getAccountType()) {
				return -1;
			}
			if (accountType < bankingDetails.getAccountType()) {
				return 1;
			}
			if (accountNumber.compareTo(bankingDetails.getAccountNumber())
				!= 0) {
				return accountNumber.compareTo(
					bankingDetails.getAccountNumber());
			}
			return 0;
		}
		return this.getClass().getName().compareTo(obj.getClass().getName());
	}

	public static final String CFG_ACCOUNT_NUMBER = "AccountNumber";
	public static final String CFG_ACCOUNT_TYPE = "AccountType";
	public static final String CFG_BANK = "Bank";
	public static final String CFG_BRANCH_CODE = "BranchCode";

	/**
	 * @see hhenson.configuration.Configurable#cfgLoad(Configuration)
	 */
	public void cfgLoad(Configuration config) {
		// <AccountNumber>11111111</AccountNumber>
		// <AccountType>AccountType<AccountType>
		// <Bank>BankName</Bank>
		// <BranchCode>0000</BranchCode>
		accountNumber = (String) config.getValue(CFG_ACCOUNT_NUMBER);
		String type = (String) config.getValue(CFG_ACCOUNT_TYPE);
		if (type != null) {
			accountType = Integer.parseInt(type);
		}
		bank = (String) config.getValue(CFG_BANK);
		branchCode = (String) config.getValue(CFG_BRANCH_CODE);
	}

	/**
	 * @see hhenson.configuration.Configurable#cfgStore()
	 */
	public Configuration cfgStore() {
		Configuration conf = Configuration.newInstance();
		conf.setValue(CFG_ACCOUNT_NUMBER, accountNumber);
		conf.setValue(CFG_ACCOUNT_TYPE, Integer.toString(accountType));
		conf.setValue(CFG_BANK, bank);
		conf.setValue(CFG_BRANCH_CODE, branchCode);
		return conf;
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return accountNumber.hashCode()
			^ accountType
			^ bank.hashCode()
			^ branchCode.hashCode();
	}

}
