package hhenson.oodb.entity;

import goodsjpi.Persistent;
import hhenson.configuration.*;
import hhenson.core.*;
import hhenson.oodb.core.*;
import java.util.*;

/**
 * Represents a legal entity that can exist.
 * @author Howard Henson
 */
public abstract class Entity
	extends Persistent
	implements Comparable, Configurable, Identifiable {

	/**
	 * This should be set with an id that is unique within
	 * the collection of entities.
	 */
	private ID id;

	/**
	 * The name of the entity.
	 * This should be used when displaying the name of the
	 * entity.
	 */
	private String name;

	/**
	 * The address that is used for all legal correspondence with the
	 * entitry.
	 */
	private Address domiciliumCitandiEtExecutandi;

	/**
	 * The default voice contact number.
	 * This is the default telephone number that the entity can be
	 * contacted on for voice communications.
	 */
	private TelephoneNumber defaultVoiceContactNumber;

	/**
	 * The default contact number for FAX communications.
	 */
	private TelephoneNumber defaultFaxContactNumber;

	/**
	 * The default email address to be used to contact this entity.
	 */
	private EmailAddress defaultEmailAddress;

	/**
	 * Default banking details.
	 */
	private BankingDetails defaultBankingDetails;

	/**
	 * The date the entity came into being.
	 * This can mean when the entity was registered, cf a company
	 * or in terms of a person could imply the birth date.
	 */
	private Date commencementDate;

	/**
	 * The date the entity is no longer.
	 * In the case of a company this would be the closing down date,
	 * and in the case of a person it would be
	 * interpreted as a deathDate.
	 * This is not to be interpreted as a scheduled execution date.
	 */
	private Date terminationDate;

	/**
	 * The tax/accounting year end of the entity.
	 * This property may not be used in a person, although
	 * all entities have these legal stigma attached to
	 * them somewhere.
	 */
	private Date taxYearEndDate;

	/**
	 * The tax number associated with an entity.
	 * This property should be applicable when an entity
	 * has an associated taxYearEndDate.
	 */
	private CommissionerOfInlandRevenueNumber taxNumber;

	/**
	 * The current active agreements that an entity has entered in to.
	 * These are the current active agreements, that is the rules
	 * of the agreements here are still to be checked and applied.
	 */
	private Map currentActiveAgreements;

	/**
	 * A historical collection of agreements that an entity has entered into.
	 * These rules are no longer active and the information provided is
	 * for historical records only.
	 */
	private Map historicalAgreements;

	/**
	 * Constructor for Entity.
	 */
	public Entity() {
		super();
	}

	/**
	 * Constructs an entity with all its useful information.
	 */
	public Entity(ID id, String name) {
		super();
		setId(id);
		setName(name);
	}

	/**
	 * Returns the id.
	 * @return ID
	 */
	public ID getId() {
		return id;
	}

	/**
	 * Sets the id.
	 * @param id The id to set
	 */
	protected void setId(ID id) {
		this.id = id;
	}

	/**
	 * Returns the name.
	 * This should be used when displaying the name of the
	 * entity.
	 * @return String
	 */
	public String getName() {
		return name;
	}

	/**
	 * Sets the name.
	 * @param name The name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Returns the domiciliumCitandiEtExecutandi.
	 * @return Address
	 */
	public Address getDomiciliumCitandiEtExecutandi() {
		return domiciliumCitandiEtExecutandi;
	}

	/**
	 * Determines if a domiciliumCitandiEtExecutandi is setup.
	 */
	public boolean hasDomiciliumCitandiEtExecutandi() {
		return domiciliumCitandiEtExecutandi != null;
	}

	/**
	 * Sets the domiciliumCitandiEtExecutandi.
	 * @param domiciliumCitandiEtExecutandi The domiciliumCitandiEtExecutandi to set
	 */
	public void setDomiciliumCitandiEtExecutandi(Address domiciliumCitandiEtExecutandi) {
		this.domiciliumCitandiEtExecutandi = domiciliumCitandiEtExecutandi;
	}

	/**
	 * Returns the defaultEmailAddress.
	 * @return EmailAddress
	 */
	public EmailAddress getDefaultEmailAddress() {
		return defaultEmailAddress;
	}

	/**
	 * determines if the default email address is set.
	 */
	public boolean hasDefaultEmailAddress() {
		return defaultEmailAddress != null;
	}

	/**
	 * Returns the defaultFaxContactNumber.
	 * @return TelephoneNumber
	 */
	public TelephoneNumber getDefaultFaxContactNumber() {
		return defaultFaxContactNumber;
	}

	/**
	 * determines if the entity has a default fax number.
	 */
	public boolean hasDefaultFaxNumber() {
		return defaultFaxContactNumber != null;
	}

	/**
	 * Returns the defaultVoiceContactNumber.
	 * @return TelephoneNumber
	 */
	public TelephoneNumber getDefaultVoiceContactNumber() {
		return defaultVoiceContactNumber;
	}

	/**
	 * determines if a default voice number is set.
	 */
	public boolean hasDefaultVoiceContactNumber() {
		return defaultVoiceContactNumber != null;
	}

	/**
	 * Sets the defaultEmailAddress.
	 * @param defaultEmailAddress The defaultEmailAddress to set
	 */
	public void setDefaultEmailAddress(EmailAddress defaultEmailAddress) {
		this.defaultEmailAddress = defaultEmailAddress;
	}

	/**
	 * Sets the defaultFaxContactNumber.
	 * @param defaultFaxContactNumber The defaultFaxContactNumber to set
	 */
	public void setDefaultFaxContactNumber(TelephoneNumber defaultFaxContactNumber) {
		this.defaultFaxContactNumber = defaultFaxContactNumber;
	}

	/**
	 * Sets the defaultVoiceContactNumber.
	 * @param defaultVoiceContactNumber The defaultVoiceContactNumber to set
	 */
	public void setDefaultVoiceContactNumber(TelephoneNumber defaultVoiceContactNumber) {
		this.defaultVoiceContactNumber = defaultVoiceContactNumber;
	}

	/**
	 * Returns the defaultBankingDetails.
	 * @return BankingDetails
	 */
	public BankingDetails getDefaultBankingDetails() {
		return defaultBankingDetails;
	}

	/**
	 * determines if the default banking details are available.
	 */
	public boolean hasDefaultBankingDetails() {
		return defaultBankingDetails != null;
	}

	/**
	 * Sets the defaultBankingDetails.
	 * @param defaultBankingDetails The defaultBankingDetails to set
	 */
	public void setDefaultBankingDetails(BankingDetails defaultBankingDetails) {
		this.defaultBankingDetails = defaultBankingDetails;
	}

	/**
	 * Returns the commencementDate.
	 * @return Date
	 */
	public Date getCommencementDate() {
		return commencementDate;
	}

	/**
	 * Sets the commencementDate.
	 * @param commencementDate The commencementDate to set
	 */
	public void setCommencementDate(Date commencementDate) {
		this.commencementDate = commencementDate;
	}

	/**
	 * Returns the taxYearEndDate.
	 * @return Date
	 */
	public Date getTaxYearEndDate() {
		return taxYearEndDate;
	}

	/**
	 * Sets the taxYearEndDate.
	 * @param taxYearEndDate The taxYearEndDate to set
	 */
	public void setTaxYearEndDate(Date taxYearEndDate) {
		this.taxYearEndDate = taxYearEndDate;
	}

	/**
	 * @see java.lang.Comparable#compareTo(Object)
	 */
	public int compareTo(Object obj) {
		if (obj instanceof Entity) {
			return id.compareTo(((Entity) obj).getId());
		}
		return this.getClass().getName().compareTo(obj.getClass().getName());
	}

	public static final String CFG_COMMENCEMENT_DATE = "CommencementDate";
	public static final String CFG_CURRENT_ACTIVE_AGREEMENTS =
		"CurrentActiveAgreements";
	public static final String CFG_BANKING_DETAILS = "BankDetails";
	public static final String CFG_EMAIL_ADDRESS = "EmailAddress";
	public static final String CFG_FAX_CONTACT_NUMBER = "FaxContactNumber";
	public static final String CFG_VOICE_CONTACT_NUMBER = "VoiceContactNumber";
	public static final String CFG_DOMICILIUM_CITANDI_EXECUTANDI =
		"domiciliumCitandiEtExecutandi";
	public static final String CFG_HISTORICAL_AGREEMENTS =
		"HistoricalAgreements";
	public static final String CFG_ID = "Id";
	public static final String CFG_NAME = "Name";
	public static final String CFG_TAX_NUMBER = "TaxNumber";
	public static final String CFG_TAX_YEAR_END_DATE = "TaxYearEndDate";
	public static final String CFG_TERMINATION_DATE = "TerminationDate";

	/**
	 * @see hhenson.configuration.Configurable#cfgLoad(Configuration)
	 */
	public void cfgLoad(Configuration config) {
		commencementDate = (Date) config.getValue(CFG_COMMENCEMENT_DATE);
		currentActiveAgreements =
			(Map) config.getValue(CFG_CURRENT_ACTIVE_AGREEMENTS);
		defaultBankingDetails =
			(BankingDetails) config.getValue(CFG_BANKING_DETAILS);
		defaultEmailAddress = (EmailAddress) config.getValue(CFG_EMAIL_ADDRESS);
		defaultFaxContactNumber =
			(TelephoneNumber) config.getValue(CFG_FAX_CONTACT_NUMBER);
		defaultVoiceContactNumber =
			(TelephoneNumber) config.getValue(CFG_VOICE_CONTACT_NUMBER);
		domiciliumCitandiEtExecutandi =
			(Address) config.getValue(CFG_DOMICILIUM_CITANDI_EXECUTANDI);
		historicalAgreements = (Map) config.getValue(CFG_HISTORICAL_AGREEMENTS);
		id = (ID) config.getValue(CFG_ID);
		name = (String) config.getValue(CFG_NAME);
		taxNumber =
			(CommissionerOfInlandRevenueNumber) config.getValue(CFG_TAX_NUMBER);
		taxYearEndDate = (Date) config.getValue(CFG_TAX_YEAR_END_DATE);
		terminationDate = (Date) config.getValue(CFG_TERMINATION_DATE);
	}

	/**
	 * @see hhenson.configuration.Configurable#cfgStore()
	 */
	public Configuration cfgStore() {
		Configuration config = Configuration.newInstance();
		config.setValue(CFG_COMMENCEMENT_DATE, commencementDate);
		config.setValue(CFG_CURRENT_ACTIVE_AGREEMENTS, currentActiveAgreements);
		config.setValue(CFG_BANKING_DETAILS, defaultBankingDetails);
		config.setValue(CFG_EMAIL_ADDRESS, defaultEmailAddress);
		config.setValue(CFG_FAX_CONTACT_NUMBER, defaultFaxContactNumber);
		config.setValue(CFG_VOICE_CONTACT_NUMBER, defaultVoiceContactNumber);
		config.setValue(
			CFG_DOMICILIUM_CITANDI_EXECUTANDI,
			domiciliumCitandiEtExecutandi);
		config.setValue(CFG_HISTORICAL_AGREEMENTS, historicalAgreements);
		config.setValue(CFG_ID, id);
		config.setValue(CFG_NAME, name);
		config.setValue(CFG_TAX_NUMBER, taxNumber);
		config.setValue(CFG_TAX_YEAR_END_DATE, taxYearEndDate);
		config.setValue(CFG_TERMINATION_DATE, terminationDate);
		return config;

	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object obj) {
		if (obj instanceof Entity) {
			return id.equals(((Entity) obj).getId());
		}
		return false;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return id.toString();
	}

	/**
	 * Returns the currentActiveAgreements.
	 * @return Map
	 */
	public Map getCurrentActiveAgreements() {
		return currentActiveAgreements;
	}

	/**
	 * Returns the historicalAgreements.
	 * @return Map
	 */
	public Map getHistoricalAgreements() {
		return historicalAgreements;
	}

	/**
	 * Returns the taxNumber.
	 * @return CommissionerOfInlandRevenueNumber
	 */
	public CommissionerOfInlandRevenueNumber getTaxNumber() {
		return taxNumber;
	}

	/**
	 * Returns the terminationDate.
	 * @return Date
	 */
	public Date getTerminationDate() {
		return terminationDate;
	}

	/**
	 * Sets the currentActiveAgreements.
	 * @param currentActiveAgreements The currentActiveAgreements to set
	 */
	public void setCurrentActiveAgreements(Map currentActiveAgreements) {
		this.currentActiveAgreements = currentActiveAgreements;
	}

	/**
	 * Sets the historicalAgreements.
	 * @param historicalAgreements The historicalAgreements to set
	 */
	public void setHistoricalAgreements(Map historicalAgreements) {
		this.historicalAgreements = historicalAgreements;
	}

	/**
	 * Sets the taxNumber.
	 * @param taxNumber The taxNumber to set
	 */
	public void setTaxNumber(CommissionerOfInlandRevenueNumber taxNumber) {
		this.taxNumber = taxNumber;
	}

	/**
	 * Sets the terminationDate.
	 * @param terminationDate The terminationDate to set
	 */
	public void setTerminationDate(Date terminationDate) {
		this.terminationDate = terminationDate;
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return id.hashCode();
	}

}
