package hhenson.oodb.entity;

import hhenson.configuration.Configuration;
import hhenson.core.ID;
import hhenson.roles.Role;
import java.util.Date;
import java.util.Set;

/**
 * A representation of a person.
 * A person has some basic differences to a generic entity
 * such as FirstName, Surname, Postal and Physical address.
 * 
 * @author Howard Henson
 */
public class Person extends Entity {

	public static final byte MALE = 'm';
	public static final byte FEMALE = 'f';
	public static final byte UNDEFINED_SEX = 'u';

	/**
	 * Indicates the sex of the entity.
	 * There are three possiblities, namely:<b>
	 * <ul><li>Male</li>
	 * <li>Female</li>
	 * <li>Undefined</li>
	 */
	private byte sex;

	/**
	 * determines if the postal address is deemed
	 * to be the Domicilium Citandi Et Executandi.
	 * By default this is set to true.
	 */
	private boolean postalAddressDefaultDomicilium = true;

	/**
	 * the physical address of the person.
	 */
	private Address physicalAddress;

	/**
	 * the postal address of the person.
	 */
	private Address postalAddress;

	/**
	 * Title of the person
	 */
	private String title;

	/**
	 * First name of the person
	 */
	private String firstName;

	/**
	 * Family name of the person
	 */
	private String familyName;

	/**
	 * Other names (middle names) of the person.
	 */
	private String otherNames;

	/**
	 * The place where the person was born.
	 * This is not a validated field and so
	 * care must be taken to ensur correct values
	 * are placed in this field.
	 */
	private String placeOfBirth;

	/**
	 * This is a number representing the nationality of the person.
	 * The number is the index of the CountryCodes array representing
	 * the country of nationality.
	 */
	private int nationality;

	/**
	 * The roles that a person is and has been associated with over
	 * time.
	 */
	private Set roles;

	/**
	 * Constructor for Person.
	 */
	public Person() {
		super();
	}

	/**
	 * Constructor for Person.
	 * @param id
	 * @param name
	 */
	public Person(ID id, String name) {
		super(id, name);
	}

	/**
	 * Returns the postalDefaultDomicilium.
	 * @return boolean
	 */
	public boolean isPostalAddressDefaultDomicilium() {
		return postalAddressDefaultDomicilium;
	}

	/**
	 * Sets the postalDefaultDomicilium.
	 * @param postalDefaultDomicilium The postalDefaultDomicilium to set
	 */
	public void setPostalAddressDefaultDomicilium(boolean postalAddressDefaultDomicilium) {
		this.postalAddressDefaultDomicilium = postalAddressDefaultDomicilium;
		if (postalAddressDefaultDomicilium) {
			//ensure postaladdress is set to default
		} else {
			//ensure physical is set to default
		}
	}

	/**
	 * Returns the physicalAddress.
	 * @return Address
	 */
	public Address getPhysicalAddress() {
		return physicalAddress;
	}

	/**
	 * Returns the postalAddress.
	 * @return Address
	 */
	public Address getPostalAddress() {
		return postalAddress;
	}

	/**
	 * Sets the physicalAddress.
	 * @param physicalAddress The physicalAddress to set
	 */
	public void setPhysicalAddress(Address physicalAddress) {
		this.physicalAddress = physicalAddress;
		if (!postalAddressDefaultDomicilium) {
			setDomiciliumCitandiEtExecutandi(physicalAddress);
		}
	}

	/**
	 * Sets the postalAddress.
	 * @param postalAddress The postalAddress to set
	 */
	public void setPostalAddress(Address postalAddress) {
		this.postalAddress = postalAddress;
		if (postalAddressDefaultDomicilium) {
			setDomiciliumCitandiEtExecutandi(postalAddress);
		}
	}

	/**
	 * Returns the familyName.
	 * @return String
	 */
	public String getFamilyName() {
		return familyName;
	}

	/**
	 * Returns the firstName.
	 * @return String
	 */
	public String getFirstName() {
		return firstName;
	}

	/**
	 * Returns the otherNames.
	 * @return String
	 */
	public String getOtherNames() {
		return otherNames;
	}

	/**
	 * Returns the title.
	 * @return String
	 */
	public String getTitle() {
		return title;
	}

	/**
	 * Sets the familyName.
	 * @param familyName The familyName to set
	 */
	public void setFamilyName(String familyName) {
		this.familyName = familyName;
	}

	/**
	 * Sets the firstName.
	 * @param firstName The firstName to set
	 */
	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	/**
	 * Sets the otherNames.
	 * @param otherNames The otherNames to set
	 */
	public void setOtherNames(String otherNames) {
		this.otherNames = otherNames;
	}

	/**
	 * Sets the title.
	 * @param title The title to set
	 */
	public void setTitle(String title) {
		this.title = title;
	}

	/**
	 * Returns the name of the person using the default formatting template.
	 * The default template is <title> <familyName>, <firstName> <otherNames>
	 * @see hhenson.oodb.entity.Entity#getName()
	 */
	public String getName() {
		return ((title != null) ? title : "")
			+ ((title != null
				&& (familyName != null
					|| firstName != null
					|| otherNames != null))
				? " "
				: "")
			+ ((familyName != null) ? familyName : "")
			+ ((familyName != null && (firstName != null || otherNames != null))
				? ", "
				: "")
			+ ((firstName != null) ? firstName : "")
			+ ((firstName != null && otherNames != null) ? " " : "")
			+ ((otherNames != null) ? otherNames : "");
	}

	/**
	 * Do not use this method with the person class as it
	 * does not do anything!
	 * @see hhenson.oodb.entity.Entity#setName(String)
	 */
	public void setName(String name) {
	}

	/**
	 * Returns the birth date of the person.
	 * This is the entity property commencementDate.
	 */
	public Date getBirthDate() {
		return getCommencementDate();
	}

	/**
	 * Sets the persons birth date of the person.
	 * This is the entity property commencementDate.
	 */
	public void setBirthDate(Date birthDate) {
		setCommencementDate(birthDate);
	}
	/**
	 * Returns the nationality.
	 * @return int
	 */
	public int getNationality() {
		return nationality;
	}

	/**
	 * Returns the placeOfBirth.
	 * @return String
	 */
	public String getPlaceOfBirth() {
		return placeOfBirth;
	}

	/**
	 * Sets the nationality.
	 * @param nationality The nationality to set
	 */
	public void setNationality(int nationality) {
		this.nationality = nationality;
	}

	/**
	 * Sets the placeOfBirth.
	 * @param placeOfBirth The placeOfBirth to set
	 */
	public void setPlaceOfBirth(String placeOfBirth) {
		this.placeOfBirth = placeOfBirth;
	}

	/**
	 * Returns the roles.
	 * @return Set
	 */
	public Set getRoles() {
		return roles;
	}

	/**
	 * Returns the sex.
	 * @return byte
	 */
	public byte getSex() {
		return sex;
	}

	/**
	 * Sets the roles.
	 * @param roles The roles to set
	 */
	public void setRoles(Set roles) {
		this.roles = roles;
	}
	/**
	 * Adds a role into the set of roles
	 * @param roles The roles to set
	 */
	public void addRole(Role role) {
		roles.add(role);
	}

	/**
	 * Sets the sex.
	 * @param sex The sex to set
	 */
	public void setSex(byte sex) {
		this.sex = sex;
	}

	public static final String CFG_SEX = "Sex";
	public static final String CFG_POSTAL_ADDRESS_DEFAULT =
		"DefaultPostalAddress";
	public static final String CFG_PHYSICAL_ADDRESS = "PhysicalAddress";
	public static final String CFG_POSTAL_ADDRESS = "PostalAddress";
	public static final String CFG_TITLE = "Title";
	public static final String CFG_FIRST_NAME = "FirstName";
	public static final String CFG_FAMILY_NAME = "FamilyName";
	public static final String CFG_OTHER_NAMES = "OtherNames";
	public static final String CFG_PLACE_OF_BIRTH = "PlaceOfBirth";
	public static final String CFG_NATIONALITY = "Nationality";
	public static final String CFG_ROLES = "Roles";
	/**
	 * @see hhenson.configuration.Configurable#cfgLoad(Configuration)
	 */
	public void cfgLoad(Configuration config) {
		super.cfgLoad(config);
		String temp = (String) config.getValue(CFG_SEX);
		if (temp != null) {
			sex = Byte.parseByte(temp);
		} else {
			sex = UNDEFINED_SEX;
		}
		temp = (String) config.getValue(CFG_POSTAL_ADDRESS_DEFAULT);
		if (temp != null) {
			postalAddressDefaultDomicilium = Boolean.getBoolean(temp);
		}
		physicalAddress = (Address) config.getValue(CFG_PHYSICAL_ADDRESS);
		postalAddress = (Address) config.getValue(CFG_POSTAL_ADDRESS);
		title = (String) config.getValue(CFG_TITLE);
		firstName = (String) config.getValue(CFG_FIRST_NAME);
		familyName = (String) config.getValue(CFG_FAMILY_NAME);
		otherNames = (String) config.getValue(CFG_OTHER_NAMES);
		placeOfBirth = (String) config.getValue(CFG_PLACE_OF_BIRTH);
		temp = (String) config.getValue(CFG_NATIONALITY);
		if (temp != null) {
			nationality = Integer.parseInt(temp);
		}
		roles = (Set) config.getValue(CFG_ROLES);

	}

	/**
	 * @see hhenson.configuration.Configurable#cfgStore()
	 */
	public Configuration cfgStore() {
		Configuration config = super.cfgStore();
		config.setValue(CFG_SEX, Byte.toString(sex));
		config.setValue(
			CFG_POSTAL_ADDRESS_DEFAULT,
			Boolean.toString(postalAddressDefaultDomicilium));
		config.setValue(CFG_PHYSICAL_ADDRESS, physicalAddress);
		config.setValue(CFG_POSTAL_ADDRESS, postalAddress);
		config.setValue(CFG_TITLE, title);
		config.setValue(CFG_FIRST_NAME, firstName);
		config.setValue(CFG_FAMILY_NAME, familyName);
		config.setValue(CFG_OTHER_NAMES, otherNames);
		config.setValue(CFG_PLACE_OF_BIRTH, placeOfBirth);
		config.setValue(CFG_NATIONALITY, Integer.toString(nationality));
		config.setValue(CFG_ROLES, roles);
		return config;
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return super.hashCode();
	}

}
