package hhenson.oodb.entity;

import goodsjpi.*;
import hhenson.configuration.*;

/**
 * Represents a telephone number.
 * A telephone number is not a standalone entity, but
 * will compare on the whole value. If telephone numbers
 * need to be tracked then the number needs an external
 * identification number to uniquely identify it.
 * 
 * @author Howard Henson
 */
public class TelephoneNumber
	extends Persistent
	implements Comparable, Configurable {

	/**
	 * The country code associated with the telephone number.
	 * Defaults to South Africa.
	 */
	private int countryCode = CountryCodes.ZA;

	/**
	 * The area code for the number. This should
	 * exclude dial codes for long distance. For example
	 * in South Africa the Jhb area code is 11, but one
	 * would dial 011 when dialing the area code.
	 * This should be set with 11. (This may be
	 * replaced with a string at some point in time).
	 */
	private int areaCode;

	/**
	 * The number of the exchange. This is usually the
	 * first 2 or 3 digits of the number excluding the
	 * country and area codes.
	 */
	private String exchangeNumber;

	/**
	 * The number within the exchange identifing your specific unit.
	 */
	private String unitNumber;

	/**
	 * The extension number of the person.
	 * This number may not exist if this is a direct line.
	 */
	private String extensionNumber;

	/**
	 * Constructor for TelephoneNumber.
	 */
	public TelephoneNumber() {
		super();
	}

	/**
	 * Returns the areaCode.
	 * @return int
	 */
	public int getAreaCode() {
		return areaCode;
	}

	/**
	 * Returns the countryCode.
	 * @return int
	 */
	public int getCountryCode() {
		return countryCode;
	}

	/**
	 * Returns the exchangeNumber.
	 * @return String
	 */
	public String getExchangeNumber() {
		return exchangeNumber;
	}

	/**
	 * Returns the extensionNumber.
	 * @return String
	 */
	public String getExtensionNumber() {
		return extensionNumber;
	}

	/**
	 * Returns the unitNumber.
	 * @return String
	 */
	public String getUnitNumber() {
		return unitNumber;
	}

	/**
	 * Sets the areaCode.
	 * @param areaCode The areaCode to set
	 */
	public void setAreaCode(int areaCode) {
		this.areaCode = areaCode;
	}

	/**
	 * Sets the countryCode.
	 * @param countryCode The countryCode to set
	 */
	public void setCountryCode(int countryCode) {
		this.countryCode = countryCode;
	}

	/**
	 * Sets the exchangeNumber.
	 * @param exchangeNumber The exchangeNumber to set
	 */
	public void setExchangeNumber(String exchangeNumber) {
		this.exchangeNumber = exchangeNumber;
	}

	/**
	 * Sets the extensionNumber.
	 * @param extensionNumber The extensionNumber to set
	 */
	public void setExtensionNumber(String extensionNumber) {
		this.extensionNumber = extensionNumber;
	}

	/**
	 * Sets the unitNumber.
	 * @param unitNumber The unitNumber to set
	 */
	public void setUnitNumber(String unitNumber) {
		this.unitNumber = unitNumber;
	}

	/**
	 * @see java.lang.Comparable#compareTo(Object)
	 */
	public int compareTo(Object obj) {
		if (obj instanceof TelephoneNumber) {
			TelephoneNumber number = (TelephoneNumber) obj;
			if (countryCode > number.getCountryCode()) {
				return -1;
			}
			if (countryCode < number.getCountryCode()) {
				return 1;
			}
			if (areaCode > number.getAreaCode()) {
				return -1;
			}
			if (areaCode < number.getAreaCode()) {
				return 1;
			}
			if (exchangeNumber.compareTo(number.getExchangeNumber()) != 0) {
				return exchangeNumber.compareTo(number.getExchangeNumber());
			}

			if (unitNumber.compareTo(number.getUnitNumber()) != 0) {
				return unitNumber.compareTo(number.getUnitNumber());
			}
			//this is because there is not definitely an extension number
			if (extensionNumber != null
				&& extensionNumber.compareTo(number.getExtensionNumber()) != 0) {
				return extensionNumber.compareTo(number.getExtensionNumber());
			}
			return 0;
		}
		return this.getClass().getName().compareTo(obj.getClass().getName());
	}

	public static final String CFG_COUNTRY_CODE = "CountryCode";
	public static final String CFG_AREA_CODE = "AreaCode";
	public static final String CFG_EXCHANGE_NUMBER = "ExchangeNumber";
	public static final String CFG_UNIT_NUMBER = "UnitNumber";
	public static final String CFG_EXTENSION_NUMBER = "ExtensionNumber";
	/**
	 * @see hhenson.configuration.Configurable#cfgLoad(Configuration)
	 */
	public void cfgLoad(Configuration config) {
		String temp = (String) config.getValue(CFG_COUNTRY_CODE);
		if (temp != null) {
			countryCode = Integer.parseInt(temp);
		}
		temp = (String) config.getValue(CFG_AREA_CODE);
		if (temp != null) {
			areaCode = Integer.parseInt(temp);
		}
		exchangeNumber = (String) config.getValue(CFG_EXCHANGE_NUMBER);
		unitNumber = (String) config.getValue(CFG_UNIT_NUMBER);
		extensionNumber = (String) config.getValue(CFG_EXTENSION_NUMBER);
	}

	/**
	 * @see hhenson.configuration.Configurable#cfgStore()
	 */
	public Configuration cfgStore() {
		Configuration config = Configuration.newInstance();
		config.setValue(CFG_COUNTRY_CODE, Integer.toString(countryCode));
		config.setValue(CFG_COUNTRY_CODE, Integer.toString(areaCode));
		config.setValue(CFG_EXCHANGE_NUMBER, exchangeNumber);
		config.setValue(CFG_UNIT_NUMBER, unitNumber);
		config.setValue(CFG_EXTENSION_NUMBER, extensionNumber);
		return config;
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object obj) {
		if (obj instanceof TelephoneNumber) {
			TelephoneNumber number = (TelephoneNumber) obj;
			if (countryCode != number.getCountryCode()) {
				return false;
			}
			if (areaCode != number.getAreaCode()) {
				return false;
			}
			if (!exchangeNumber.equals(number.getExchangeNumber())) {
				return false;
			}

			if (!unitNumber.equals(number.getUnitNumber())) {
				return false;
			}
			if (extensionNumber != null
				&& !extensionNumber.equals(number.getExtensionNumber())) {
				return false;
			}
			return true;
		}
		return false;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuffer strBuffer = new StringBuffer();
		strBuffer.append(countryCode);
		strBuffer.append(areaCode);
		strBuffer.append(exchangeNumber);
		strBuffer.append(unitNumber);
		if (extensionNumber != null) {
			strBuffer.append(extensionNumber);
		}
		return strBuffer.toString();
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return countryCode
			^ areaCode
			^ exchangeNumber.hashCode()
			^ unitNumber.hashCode()
			^ extensionNumber.hashCode();
	}

}
