package hhenson.oodb.entity.company;

import hhenson.configuration.Configuration;
import hhenson.core.ID;
import hhenson.oodb.entity.Entity;
import hhenson.oodb.rules.RuleCollection;
import java.util.Map;
import java.util.Set;

/**
 * A company is an entity that has company structures associated with it.
 * Examples of structure include branches, etc.
 * 
 * @author Howard Henson
 */
public class Company extends Entity {

	/**
	 * The current active branches that are associated with the company.
	 * These contain CompanyBranch objects.
	 */
	private Set currentBranches;

	/**
	 * The paypoints associated with a company.
	 * These contain CompanyPayPoint objects.
	 */
	private Set currentPayPoints;

	/**
	 * The cost cetres associated with the company.
	 * The cost centres are the units where employees are
	 * assocated and costs are aggregated.
	 */
	private Set costCentres;

	/**
	 * The rules associated with a company.
	 * These may be rules like: Tax rules, employment rules,
	 * accounting rules, legal rules, etc.
	 */
	private RuleCollection companyRules;

	/**
	 * maps the category code to the category description.
	 */
	//       |
	//    +--+--+
	// catCode
	//    |
	// description (String)
	private Map employeeCategoryMap;

	/**
	 * The tangable and non-tangable offerings that is
	 * offered to potential clients and consumers. This
	 * is a set of Product objects.
	 */
	private Set productOfferings;

	/**
	 * A company that provides products in the financial services
	 * industry.
	 */
	public static final int FINANCIAL_SERVICES = 1;

	/**
	 * A company that provides products or services in the
	 * technology sector.
	 */
	public static final int INFORMATION_TECHNOLOGY = 2;

	/**
	 * Provides insurance related services.
	 */
	public static final int INSURANCE = 4;

	/**
	 * Provides heath care services.
	 */
	public static final int HEALTH_CARE = 5;

	/**
	 * The type of industry the company is associated with.
	 */
	private int industryType;

	/**
	 * Constructor for Company.
	 */
	public Company() {
		super();
	}

	/**
	 * Constructor for Company.
	 * @param id
	 * @param name
	 */
	public Company(ID id, String name) {
		super(id, name);
	}

	/**
	 * Returns the companyRules.
	 * @return RuleCollection
	 */
	public RuleCollection getCompanyRules() {
		return companyRules;
	}

	/**
	 * Returns the costCentres.
	 * @return Set
	 */
	public Set getCostCentres() {
		return costCentres;
	}

	/**
	 * Returns the currentBranches.
	 * @return Set
	 */
	public Set getCurrentBranches() {
		return currentBranches;
	}

	/**
	 * Returns the currentPayPoints.
	 * @return Set
	 */
	public Set getCurrentPayPoints() {
		return currentPayPoints;
	}

	/**
	 * Returns the employeeCategoryMap.
	 * @return Map
	 */
	public Map getEmployeeCategoryMap() {
		return employeeCategoryMap;
	}

	/**
	 * Returns the industryType.
	 * @return int
	 */
	public int getIndustryType() {
		return industryType;
	}

	/**
	 * Returns the productOfferings.
	 * @return Set
	 */
	public Set getProductOfferings() {
		return productOfferings;
	}

	/**
	 * Sets the companyRules.
	 * @param companyRules The companyRules to set
	 */
	public void setCompanyRules(RuleCollection companyRules) {
		this.companyRules = companyRules;
	}

	/**
	 * Sets the costCentres.
	 * @param costCentres The costCentres to set
	 */
	public void setCostCentres(Set costCentres) {
		this.costCentres = costCentres;
	}

	/**
	 * Sets the costCentres.
	 * @param costCentres The costCentres to set
	 */
	public void setCostCentre(CostCentre costCentre) {
		costCentres.add(costCentre);
	}

	/**
	 * Sets the currentBranches.
	 * @param currentBranches The currentBranches to set
	 */
	public void setCurrentBranches(Set currentBranches) {
		this.currentBranches = currentBranches;
	}

	/**
	 * Adds a branch the currentBranches.
	 * @param currentBranches The currentBranches to set
	 */
	public void setCurrentBranch(CompanyBranch companyBranch) {
		currentBranches.add(companyBranch);
	}

	/**
	 * Sets the currentPayPoints.
	 * @param currentPayPoints The currentPayPoints to set
	 */
	public void setCurrentPayPoints(Set currentPayPoints) {
		this.currentPayPoints = currentPayPoints;
	}

	/**
	* Sets the currentPayPoints.
	* @param currentPayPoints The currentPayPoints to set
	*/
	public void setCurrentPayPoint(CompanyPayPoint companyPayPoint) {
		currentPayPoints.add(companyPayPoint);
	}

	/**
	 * Sets the employeeCategoryMap.
	 * @param employeeCategoryMap The employeeCategoryMap to set
	 */
	public void setEmployeeCategoryMap(Map employeeCategoryMap) {
		this.employeeCategoryMap = employeeCategoryMap;
	}
	/**
	 * Adds to the employeeCategoryMap.
	 * @param employeeCategoryMap The employeeCategoryMap to set
	 */
	public void addEmployeeCategory(Object key, Object value) {
		employeeCategoryMap.put(key, value);
	}

	/**
	 * Sets the industryType.
	 * @param industryType The industryType to set
	 */
	public void setIndustryType(int industryType) {
		this.industryType = industryType;
	}

	/**
	 * Sets the productOfferings.
	 * @param productOfferings The productOfferings to set
	 */
	public void setProductOfferings(Set productOfferings) {
		this.productOfferings = productOfferings;
	}
	/**
	 * Adds a product to the productOfferings set.
	 * @param productOfferings The productOfferings to set
	 */
	public void setProductOffering(Product product) {
		productOfferings.add(product);
	}

	public static final String CFG_CURRENT_BRANCHES = "CurrentBranches";
	public static final String CFG_CURRENT_PAYPOINTS = "CurrentPaypoints";
	public static final String CFG_COST_CENTRES = "CostCentres";
	public static final String CFG_COMPANY_RULES = "CompanyRules";
	public static final String CFG_EMPLOYEE_CATEGORY_MAP = "EmployeeCategoryMap";
	public static final String CFG_PRODUCT_OFFERINGS = "ProductOfferings";
	public static final String CFG_INDUSTRY_TYPE = "IndustryType";

	/**
	 * @see hhenson.configuration.Configurable#cfgLoad(Configuration)
	 */
	public void cfgLoad(Configuration config) {
		super.cfgLoad(config);
		currentBranches = (Set) config.getValue(CFG_CURRENT_BRANCHES);
		currentPayPoints = (Set) config.getValue(CFG_CURRENT_PAYPOINTS);
		costCentres = (Set) config.getValue(CFG_COST_CENTRES);
		companyRules = (RuleCollection) config.getValue(CFG_COMPANY_RULES);
		employeeCategoryMap = (Map) config.getValue(CFG_EMPLOYEE_CATEGORY_MAP);
		productOfferings = (Set) config.getValue(CFG_PRODUCT_OFFERINGS);
		String type = (String) config.getValue(CFG_INDUSTRY_TYPE);
		if (type != null) {
			industryType = Integer.parseInt(type);
		}
	}

	/**
	 * @see hhenson.configuration.Configurable#cfgStore()
	 */
	public Configuration cfgStore() {
		Configuration config = super.cfgStore();
		config.setValue(CFG_CURRENT_BRANCHES, currentBranches);
		config.setValue(CFG_CURRENT_PAYPOINTS, currentPayPoints);
		config.setValue(CFG_COST_CENTRES, costCentres);
		config.setValue(CFG_COMPANY_RULES, companyRules);
		config.setValue(CFG_EMPLOYEE_CATEGORY_MAP, employeeCategoryMap);
		config.setValue(CFG_PRODUCT_OFFERINGS, productOfferings);
		config.setValue(CFG_INDUSTRY_TYPE, Integer.toString(industryType)); 
		return config;
	}

}
