package hhenson.oodb.entity.company;

import hhenson.core.InvalidIDException;
import hhenson.core.ObjectNotLocatedException;
import hhenson.oodb.core.OodbIDImpl;
import hhenson.oodb.entity.CountryCodes;

/**
 * @author Howard Henson
 */
public class CompanyID extends OodbIDImpl {

	/**
	 * The company type could not be determined.
	 */
	public static final int UNKOWN_COMPANY_TYPE = 0;

	/**
	 * A flag to represent the company is a closed
	 * corporation. A closed corporation is a company
	 * consisting of individual members who carry
	 * limited liabilty for the company. Typically
	 * there are between 1 and 12 members of a cc and
	 * they have lesser audit requirements than any
	 * other type of company. The company must have
	 * there books checked by a qualified book keeper
	 * once a year.
	 */
	public static final int CLOSED_CORPORATION = 1;

	/**
	 * A private company is a PTY(LTD).
	 * This is a non listed company. The company is made
	 * up of directors of the company, who each share limited
	 * liability toward the company. The company has reasonably
	 * stringent audit requirements in that a registered charted
	 * accountant is required to review the books anually.
	 */
	public static final int PRIVATE_COMPANY = 2;

	/**
	 * A public company is a PTY.
	 * This is a listed company. The company is made up of at least
	 * 10 shareholders. The company is listed on a stock exchange
	 * and must abide by the rules set up by the exchange. These typically
	 * include that they must be audited by at least 2 external audit firms.
	 */
	public static final int PUBLIC_COMPANY = 4;

	/**
	 * A partnership is a company that is formed by individuals who are
	 * jointly and serverably liable.
	 * A partnership is dissolved and re-created each time the partner list
	 * changes or the aggrement making up the partnership changes.
	 */
	public static final int PARTNERSHIP = 8;

	/**
	 * The registration id associated with the company.
	 * This is a number typically assigned by a national
	 * registration authority.
	 */
	private String registrationNumber;

	/**
	 * The type of company represents the nature
	 * of the company that is registered.
	 * For example a closed corporations, a public company, etc.
	 * By default the ID is set up to 
	 */
	private int companyType = UNKOWN_COMPANY_TYPE;

	/**
	 * We will assume this to be the country that the company
	 * is registered in.
	 */
	private int registrationAuthority = CountryCodes.ZA;

	/**
	 * Constructor for CompanyID.
	 */
	public CompanyID() {
		super();
	}

	/**
	 * @see java.lang.Comparable#compareTo(Object)
	 */
	public int compareTo(Object o) {
		if (o instanceof CompanyID) {
			CompanyID companyID = (CompanyID) o;
			if (registrationAuthority > companyID.getRegistrationAuthority()) {
				return -1;
			}
			if (registrationAuthority < companyID.getRegistrationAuthority()) {
				return 1;
			}
			if (registrationNumber.compareTo(companyID.getRegistrationNumber())
				!= 0) {
				return registrationNumber.compareTo(
					companyID.getRegistrationNumber());
			}
			if (companyType > companyID.getCompanyType()) {
				return -1;
			}
			if (companyType < companyID.getCompanyType()) {
				return 1;
			}
			return 0;
		}
		return this.getClass().getName().compareTo(o.getClass().getName());
	}

	/**
	 * Returns the companyType.
	 * @return int
	 */
	public int getCompanyType() {
		return companyType;
	}

	/**
	 * Returns the registrationAuthority.
	 * @return int
	 */
	public int getRegistrationAuthority() {
		return registrationAuthority;
	}

	/**
	 * Returns the registrationNumber.
	 * @return String
	 */
	public String getRegistrationNumber() {
		return registrationNumber;
	}

	/**
	 * Sets the companyType.
	 * @param companyType The companyType to set
	 */
	public void setCompanyType(int companyType) {
		this.companyType = companyType;
	}

	/**
	 * Sets the registrationAuthority.
	 * @param registrationAuthority The registrationAuthority to set
	 */
	public void setRegistrationAuthority(int registrationAuthority) {
		this.registrationAuthority = registrationAuthority;
	}

	/**
	 * Sets the registrationNumber.
	 * @param registrationNumber The registrationNumber to set
	 */
	public void setRegistrationNumber(String registrationNumber) {
		this.registrationNumber = registrationNumber;
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return registrationAuthority^registrationNumber.hashCode()^companyType;
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object o) {
		if (o instanceof CompanyID) {
			CompanyID companyID = (CompanyID) o;
			if (registrationAuthority != companyID.getRegistrationAuthority()) {
				return false;
			}			
			if (!registrationNumber.equals(companyID.getRegistrationNumber())) {
				return false;
			}
			if (companyType != companyID.getCompanyType()) {
				return false;
			}			
			return true;
		}
		return false;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return super.toString();
	}

	/**
	 * @see hhenson.oodb.core.ID#fetchObjectFromDatabase()
	 */
	protected Object fetchObjectFromDatabase()
		throws InvalidIDException, ObjectNotLocatedException {
		return null;
	}

}
