package hhenson.oodb.entity.company;

import hhenson.core.ID;
import java.util.Map;

/**
 * A product is something a company offers potential clients or consumers.
 * The product instance represents the generic offering and does not define
 * an individual instance of a product.
 * 
 * The individual items of a product will different in type from product
 * to product and will be represented differently. Eg A skate board is a physical
 * items and can potentially be represented with a serial number. However a unit
 * from a unit trust is not uniquely representable.
 * 
 * @author Howard Henson
 */
public class Product {
	
	/**
	 * Determines if the elements of the product
	 * are tangable or not. Example a skate board is
	 * a tangible product where a consuling product is not.
	 */
	private boolean tangableOffering = false;

	/**
	 * The descriptive name for this product.
	 */
	private String name;
	
	/**
	 * The unique id associated with this product.
	 */
	private ID id;
	
	/**
	 * The information associated with a product.
	 * This is collected by key and contains a
	 * blob for binary objects such as PDF documents (e.g. brouchers).
	 */
	//        |
	//  +-----+-----+
	// Key (String)
	//  |
	//Value (Blob)
	private Map productStaticInformation;
	
	/**
	 * The company that provides this product offering.
	 */
	private Company company;
	
	/**
	 * Constructor for Product.
	 */
	public Product() {
		super();
	}

	/**
	 * Returns the company.
	 * @return Company
	 */
	public Company getCompany() {
		return company;
	}

	/**
	 * Returns the id.
	 * @return ID
	 */
	public ID getId() {
		return id;
	}

	/**
	 * Returns the name.
	 * @return String
	 */
	public String getName() {
		return name;
	}

	/**
	 * Returns the productStaticInformation.
	 * @return Map
	 */
	public Map getProductStaticInformation() {
		return productStaticInformation;
	}

	/**
	 * Returns the tangableOffering.
	 * @return boolean
	 */
	public boolean isTangableOffering() {
		return tangableOffering;
	}

	/**
	 * Sets the company.
	 * @param company The company to set
	 */
	public void setCompany(Company company) {
		this.company = company;
	}

	/**
	 * Sets the id.
	 * @param id The id to set
	 */
	public void setId(ID id) {
		this.id = id;
	}

	/**
	 * Sets the name.
	 * @param name The name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Sets the productStaticInformation.
	 * @param productStaticInformation The productStaticInformation to set
	 */
	public void setProductStaticInformation(Map productStaticInformation) {
		this.productStaticInformation = productStaticInformation;
	}
	
	/**
	 * Adds productStaticInformation to the map.
	 * @param productStaticInformation The productStaticInformation to set
	 */
	public void addProductStaticInformation(Object key, Object value) {
		productStaticInformation.put(key, value);
	}


	/**
	 * Sets the tangableOffering.
	 * @param tangableOffering The tangableOffering to set
	 */
	public void setTangableOffering(boolean tangableOffering) {
		this.tangableOffering = tangableOffering;
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object obj) {
		if (obj instanceof Product){
			return id.equals(((Product)obj).getId());	
		}
		return false;
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return id.hashCode();
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return id.toString();
	}

}
