package hhenson.oodb.roles;

import hhenson.core.*;
import hhenson.oodb.core.OodbIDImpl;
import hhenson.roles.Role;
import hhenson.utils.DateRange;

/**
 * Implements a basic ID that can be used to represent a role
 * object in the system.
 * 
 * This utilised some basic information about the role to generate
 * what should be a unique key.
 * 
 * @author Howard Henson
 */
public class GenericRoleID extends OodbIDImpl {

	/**
	 * The person who is associated with this role.
	 */
	private ID personID;

	/**
	 * The range that the role is applicable for.
	 * If the endDate is null, and we can't find the object
	 * we should concider that the role has terminated since
	 * generating this key and the end date is now set.
	 * Thus only match the start date of the role and ensure
	 * the end date is set.
	 */
	private DateRange dateRange;

	/**
	 * The class name of the role object that is associated
	 * with this id.
	 */
	private String roleClassName;

	/**
	 * Constructor for GenericRoleID.
	 */
	public GenericRoleID() {
		super();
	}

	/**
	 * This is the constuctor that should be
	 * used to gererate a new GenericRoleID.
	 */
	public GenericRoleID(Role role) {
		super();
		personID = role.getPerson().getId();
		dateRange = role.getValidDateRange();
		roleClassName = role.getClass().getName();
	}

	/**
	 * @see java.lang.Comparable#compareTo(Object)
	 */
	public int compareTo(Object obj) {
		if (obj instanceof GenericRoleID) {
			GenericRoleID roleID = (GenericRoleID) obj;
			if (personID.compareTo(roleID.getPersonID())!=0) {
				return personID.compareTo(roleID.getPersonID()); 
			}
			if (roleClassName.compareTo(roleID.getRoleClassName())!=0) {
				return roleClassName.compareTo(roleID.getRoleClassName()); 
			}
			if (dateRange.compareTo(roleID.getDateRange())!=0) {
				return dateRange.compareTo(roleID.getDateRange()); 
			}
			return 0;
		}
		return this.getClass().getName().compareTo(obj.getClass().getName());
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return personID.hashCode()^roleClassName.hashCode()^dateRange.hashCode();
	}

	/**
	 * Two IDs are identical if the person id AND the role is the same
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object obj) {
		if (obj instanceof GenericRoleID) {
			GenericRoleID roleID = (GenericRoleID) obj;
			if (!personID.equals(roleID.getPersonID())) {
				return false;
			}
			if (!roleClassName.equals(roleID.getRoleClassName())) {
				return false;
			}
			if (!dateRange.equals(roleID.getDateRange())) {
				return false;
			}
			return true;
		}
		return false;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return super.toString();
	}

	/**
	 * Returns the dateRange.
	 * @return DateRange
	 */
	public DateRange getDateRange() {
		return dateRange;
	}

	/**
	 * Returns the personID.
	 * @return ID
	 */
	public ID getPersonID() {
		return personID;
	}

	/**
	 * Returns the roleClassName.
	 * @return String
	 */
	public String getRoleClassName() {
		return roleClassName;
	}

	/**
	 * Sets the dateRange.
	 * @param dateRange The dateRange to set
	 */
	public void setDateRange(DateRange dateRange) {
		this.dateRange = dateRange;
	}

	/**
	 * Sets the personID.
	 * @param personID The personID to set
	 */
	protected void setPersonID(ID personID) {
		this.personID = personID;
	}

	/**
	 * Sets the roleClassName.
	 * @param roleClassName The roleClassName to set
	 */
	public void setRoleClassName(String roleClassName) {
		this.roleClassName = roleClassName;
	}

	/**
	 * @see hhenson.oodb.core.ID#fetchObjectFromDatabase()
	 */
	protected Object fetchObjectFromDatabase()
		throws InvalidIDException, ObjectNotLocatedException {
		return null;
	}

}
