package hhenson.oodb.roles;

import goodsjpi.Persistent;
import hhenson.configuration.Configurable;
import hhenson.configuration.Configuration;
import hhenson.roles.ContainedRole;
import hhenson.roles.Role;
import hhenson.utils.DateRange;

/**
 * A role contained within another role.
 * This could be used to define a concept such as roles within an employee role.
 * This type of role is not identifiable, in that it has no existance outside of a role,
 * thus it is a property of a role and can be identified as uniquely by its name within
 * the roles collection of contained roles.
 * @author Howard Henson
 */
public class OodbContainedRoleImpl
	extends Persistent
	implements ContainedRole {

	/**
	 * A sort description of the contained role.
	 */
	private String name;

	/**
	 * The role that this contained role is associated with.
	 */
	private Role associatedRole;

	/**
	 * The date range that this contained role is valid for.
	 */
	private DateRange validDateRange;

	/**
	 * Constructor for ContainedRole.
	 */
	public OodbContainedRoleImpl() {
		super();
	}

	/**
	 * Returns the associatedRole.
	 * @return Role
	 */
	public Role getAssociatedRole() {
		return associatedRole;
	}

	/**
	 * Returns the validDateRange.
	 * @return DateRange
	 */
	public DateRange getValidDateRange() {
		return validDateRange;
	}

	/**
	 * Sets the associatedRole.
	 * @param associatedRole The associatedRole to set
	 */
	public void setAssociatedRole(Role associatedRole) {
		this.associatedRole = associatedRole;
	}

	/**
	 * Sets the validDateRange.
	 * @param validDateRange The validDateRange to set
	 */
	public void setValidDateRange(DateRange validDateRange) {
		this.validDateRange = validDateRange;
	}

	/**
	 * @see java.lang.Comparable#compareTo(Object)
	 */
	public int compareTo(Object o) {
		if (o instanceof OodbContainedRoleImpl){
			OodbContainedRoleImpl role=(OodbContainedRoleImpl)o;
			if (name.compareTo(role.getName())!=0){
				return name.compareTo(role.getName());
			}
			if (associatedRole.compareTo(role.getAssociatedRole())!=0){
				return associatedRole.compareTo(role.getAssociatedRole());
			}
			if (validDateRange.compareTo(role.getValidDateRange())!=0){
				return validDateRange.compareTo(role.getValidDateRange());
			}	
			return 0;					
		}
		return this.getClass().getName().compareTo(o.getClass().getName());
	}

	public static final String CFG_NAME = "Name";
	public static final String CFG_ASSOCIATED_ROLE = "AssociatedRole";
	public static final String CFG_VALID_DATE_RANGE = "ValidDateRange";
	/**
	 * @see hhenson.configuration.Configurable#cfgLoad(Configuration)
	 */
	public void cfgLoad(Configuration config) {
		name=(String)config.getValue(CFG_NAME);
		associatedRole=(Role)config.getValue(CFG_ASSOCIATED_ROLE);
		validDateRange=(DateRange)config.getValue(CFG_VALID_DATE_RANGE);
	}

	/**
	 * @see hhenson.configuration.Configurable#cfgStore()
	 */
	public Configuration cfgStore() {
		Configuration config=Configuration.newInstance();
		config.setValue(CFG_NAME, name);
		config.setValue(CFG_ASSOCIATED_ROLE, associatedRole);
		config.setValue(CFG_VALID_DATE_RANGE, validDateRange);
		return config;
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object o) {
		if (o instanceof OodbContainedRoleImpl){
			OodbContainedRoleImpl role=(OodbContainedRoleImpl)o;
			if (!name.equals(role.getName())){
				return false;
			}
			if (!associatedRole.equals(role.getAssociatedRole())){
				return false;
			}
			if (!validDateRange.equals(role.getValidDateRange())){
				return false;
			}	
			return true;					
		}
		return false;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return super.toString();
	}

	
	/**
	 * Returns the name.
	 * @return String
	 */
	public String getName() {
		return name;
	}
	/**
	 * Sets the name.
	 * @param name The name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

}
