package hhenson.oodb.roles;

import goodsjpi.Persistent;
import hhenson.configuration.Configuration;
import hhenson.core.ID;
import hhenson.oodb.entity.Person;
import hhenson.roles.Role;
import hhenson.utils.DateRange;

import java.util.Set;

/**
 * A role represents the different things that a Person can be.
 * <p>
 * A role is always associated with an entity or in this case a person.
 * A role is a set of attributes and meanings that can be applied
 * to a person but will change over time.
 * </p>
 * <p>
 * Example, a person plays the role of child and later could plays the role
 * of parent. Also a person can be a student and later an employed person
 * or both.
 * </p>
 * 
 * @author Howard Henson
 */
public abstract class OodbRoleImpl
	extends Persistent
	implements Role {

	/**
	 * The person associated with this role.
	 */
	private Person person;

	/**
	 * The dates for which this role has meaning.
	 * For example if the role were employee, the
	 * fromDate would imply start date and the end
	 * date would be when the employee left the company
	 * or null if still employeed.
	 */
	private DateRange validDateRange;

	/**
	 * Identifies that are used by other systems to uniquely
	 * identify the instance of this role. This is probably best
	 * stored as a structured ID with the first level defining
	 * what system the id is applicable to, followed by the 
	 * second level defining the id the system uses.
	 * (CIF)
	 */
	private Set associatedIdentifications;

	/**
	 * Constructor for Role.
	 */
	public OodbRoleImpl() {
		super();
	}

	/**
	 * Returns the unique id that can be used to identify this role
	 * in the database.
	 */
	public ID getId() {
		return new GenericRoleID(this);
	}

	/**
	 * Returns the person.
	 * @return Person
	 */
	public Person getPerson() {
		return person;
	}

	/**
	 * Returns the validDateRange.
	 * @return DateRange
	 */
	public DateRange getValidDateRange() {
		return validDateRange;
	}

	/**
	 * Sets the person.
	 * @param person The person to set
	 */
	public void setPerson(Person person) {
		this.person = person;
	}

	/**
	 * Sets the validDateRange.
	 * @param validDateRange The validDateRange to set
	 */
	public void setValidDateRange(DateRange validDateRange) {
		this.validDateRange = validDateRange;
	}

	/**
	 * @see java.lang.Comparable#compareTo(Object)
	 */
	public int compareTo(Object o) {
		if (o instanceof OodbRoleImpl) {
			return this.getId().compareTo(((OodbRoleImpl) o).getId());
		}
		return this.getClass().getName().compareTo(o.getClass().getName());
	}

	public static final String PERSON="Person";
	public static final String ASSOCIATED_IDENTIFICATIONS="AssociatedIdentifications";
	public static final String VALID_DATE_RANGE="ValidDateRange";
	/**
	 * @see hhenson.configuration.Configurable#cfgLoad(Configuration)
	 */
	public void cfgLoad(Configuration config) {
		person=(Person)config.getValue(PERSON);
		associatedIdentifications=(Set)config.getValue(ASSOCIATED_IDENTIFICATIONS);
		validDateRange=(DateRange)config.getValue(VALID_DATE_RANGE);
	}

	/**
	 * @see hhenson.configuration.Configurable#cfgStore()
	 */
	public Configuration cfgStore() {
		Configuration config=Configuration.newInstance();
		config.setValue(PERSON, person);
		config.setValue(ASSOCIATED_IDENTIFICATIONS, associatedIdentifications);
		config.setValue(VALID_DATE_RANGE, validDateRange);
		return config;
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return this.getId().hashCode();
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object obj) {
		if (obj instanceof OodbRoleImpl) {
			return this.getId().equals(((OodbRoleImpl) obj).getId());
		}
		return false;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return super.toString();
	}

}
