package hhenson.oodb.roles.employee;

import hhenson.configuration.Configuration;
import hhenson.core.ID;
import hhenson.oodb.entity.EmailAddress;
import hhenson.oodb.entity.TelephoneNumber;
import hhenson.oodb.entity.company.*;
import hhenson.oodb.roles.OodbRoleImpl;
import hhenson.roles.Role;
import hhenson.utils.DateRange;

import java.util.Set;

/**
 * A role representing a person who is gainfully employeed.
 * 
 * @author Howard Henson
 */
public class Employee extends OodbRoleImpl {

	/**
	 * The business telephone contact number.
	 */
	private TelephoneNumber officeNumber;

	/**
	 * The office fax number.
	 */
	private TelephoneNumber officeFaxNumber;

	/**
	 * The office email address that can be used for
	 * business communications.
	 */
	private EmailAddress officeEMailAddress;

	/**
	 * The company the person is employed by.
	 */
	private Company company;

	/**
	 * The company branch the person is associated with.
	 * The person may or may not be associated with a branch.
	 */
	private CompanyBranch companyBranch;

	/**
	 * The paypoint a person is associated with.
	 * This property is not requried to be set.
	 */
	private CompanyPayPoint companyPayPoint;

	/**
	 * The cost centre to which this person is assigned
	 */
	private CostCentre costCentre;

	/**
	 * The salary history for the employee.
	 */
	private SalaryHistory salaryHistory;

	/**
	 * The ID associated with an employee.
	 * This is generated and maintained by the company
	 * providing the employment. This must be unique within
	 * the company.
	 */
	private EmployeeID employeeID;

	/**
	 * Denotes the employee's category within the company.
	 * Examples include TeaLady, Proffessional, MD, etc.
	 * Each company should define their own employee
	 * categories.
	 */
	private int employeeCategory;

	/**
	 * The ContainedRole that are associated with this employee.
	 * These are things like an employee could be a broker or
	 * a consultant, etc. These could be viewed as positions
	 * in the company, however this may not always be the case
	 * as an employee may have the position of Manager, but still
	 * play the role of developer when required.
	 */
	private Set roles;

	/**
	 * If set reports that the employee is currently having a break
	 * in service to the company. This could be due to conditions such
	 * as pregnancy. If this property is null, then it implies
	 * that the employee is not currently having a break in service.
	 */
	private DateRange currentBreakInService;

	/**
	 * The collection of date range objects detailing historical
	 * breaks in service.
	 */
	private Set breaksInService;

	/**
	 * Constructor for Employee.
	 */
	public Employee() {
		super();
	}

	public static final String CFG_OFFICE_NUMBER = "OfficeNumber";
	public static final String CFG_OFFICE_FAX_NUMBER = "OfficeFaxNumber";
	public static final String CFG_OFFICE_EMAIL_ADDRESS = "OfficeEmailAddress";
	public static final String CFG_COMPANY = "Company";
	public static final String CFG_COMPANY_BRANCH = "CompanyBranch";
	public static final String CFG_COMPANY_PAYPOINT = "CompanyPaypoint";
	public static final String CFG_COST_CENTRE = "CostCentre";
	public static final String CFG_SALARY_HISTORY = "SalaryHistory";
	public static final String CFG_EMPLOYEE_ID = "EmployeeID";
	public static final String CFG_EMPLOYEE_CATEGORY = "EmployeeCategory";
	public static final String CFG_ROLES = "Roles";
	public static final String CFG_CURRENT_BREAK_IN_EMPLOYEE_SERVICE =
		"CurrentBreakInService";
	public static final String CFG_EMPLOYEE_BREAK_IN_SERVICE =
		"EmployeeBreakInService";
	/**
	 * @see hhenson.configuration.Configurable#cfgLoad(Configuration)
	 */
	public void cfgLoad(Configuration config) {
		super.cfgLoad(config);
		officeNumber = (TelephoneNumber) config.getValue(CFG_OFFICE_NUMBER);
		officeFaxNumber =
			(TelephoneNumber) config.getValue(CFG_OFFICE_FAX_NUMBER);
		officeEMailAddress =
			(EmailAddress) config.getValue(CFG_OFFICE_EMAIL_ADDRESS);
		company = (Company) config.getValue(CFG_COMPANY);
		companyBranch = (CompanyBranch) config.getValue(CFG_COMPANY_BRANCH);
		companyPayPoint =
			(CompanyPayPoint) config.getValue(CFG_COMPANY_PAYPOINT);
		costCentre = (CostCentre) config.getValue(CFG_COST_CENTRE);
		salaryHistory = (SalaryHistory) config.getValue(CFG_SALARY_HISTORY);
		employeeID = (EmployeeID) config.getValue(CFG_EMPLOYEE_ID);
		String category = (String) config.getValue(CFG_EMPLOYEE_CATEGORY);
		if (category != null) {
			employeeCategory = Integer.parseInt(category);
		}
		roles = (Set) config.getValue(CFG_ROLES);
		currentBreakInService =
			(DateRange) config.getValue(CFG_CURRENT_BREAK_IN_EMPLOYEE_SERVICE);
		breaksInService = (Set) config.getValue(CFG_EMPLOYEE_BREAK_IN_SERVICE);
	}

	/**
	 * @see hhenson.configuration.Configurable#cfgStore()
	 */
	public Configuration cfgStore() {
		Configuration config = super.cfgStore();
		config.setValue(CFG_OFFICE_NUMBER, officeNumber);
		config.setValue(CFG_OFFICE_FAX_NUMBER, officeFaxNumber);
		config.setValue(CFG_OFFICE_EMAIL_ADDRESS, officeEMailAddress);
		config.setValue(CFG_COMPANY, company);
		config.setValue(CFG_COMPANY_BRANCH, companyBranch);
		config.setValue(CFG_COMPANY_PAYPOINT, companyPayPoint);
		config.setValue(CFG_COST_CENTRE, costCentre);
		config.setValue(CFG_SALARY_HISTORY, salaryHistory);
		config.setValue(CFG_EMPLOYEE_ID, employeeID);
		config.setValue(
			CFG_EMPLOYEE_CATEGORY,
			Integer.toString(employeeCategory));
		config.setValue(CFG_ROLES, roles);
		config.setValue(
			CFG_CURRENT_BREAK_IN_EMPLOYEE_SERVICE,
			currentBreakInService);
		config.setValue(CFG_EMPLOYEE_BREAK_IN_SERVICE, breaksInService);
		return config;
	}

	/**
	 * @see hhenson.oodb.roles.Role#getId()
	 */
	public ID getId() {
		return employeeID;
	}

	/**
	 * Returns the breaksInService.
	 * @return Set
	 */
	public Set getBreaksInService() {
		return breaksInService;
	}

	/**
	 * Returns the company.
	 * @return Company
	 */
	public Company getCompany() {
		return company;
	}

	/**
	 * Returns the companyBranch.
	 * @return CompanyBranch
	 */
	public CompanyBranch getCompanyBranch() {
		return companyBranch;
	}

	/**
	 * Returns the companyPayPoint.
	 * @return CompanyPayPoint
	 */
	public CompanyPayPoint getCompanyPayPoint() {
		return companyPayPoint;
	}

	/**
	 * Returns the costCentre.
	 * @return CostCentre
	 */
	public CostCentre getCostCentre() {
		return costCentre;
	}

	/**
	 * Returns the currentBreakInService.
	 * @return DateRange
	 */
	public DateRange getCurrentBreakInService() {
		return currentBreakInService;
	}

	/**
	 * Returns the employeeCategory.
	 * @return int
	 */
	public int getEmployeeCategory() {
		return employeeCategory;
	}

	/**
	 * Returns the employeeID.
	 * @return EmployeeID
	 */
	public EmployeeID getEmployeeID() {
		return employeeID;
	}

	/**
	 * Returns the officeEMailAddress.
	 * @return EmailAddress
	 */
	public EmailAddress getOfficeEMailAddress() {
		return officeEMailAddress;
	}

	/**
	 * Returns the officeFaxNumber.
	 * @return TelephoneNumber
	 */
	public TelephoneNumber getOfficeFaxNumber() {
		return officeFaxNumber;
	}

	/**
	 * Returns the officeNumber.
	 * @return TelephoneNumber
	 */
	public TelephoneNumber getOfficeNumber() {
		return officeNumber;
	}

	/**
	 * Returns the roles.
	 * @return Set
	 */
	public Set getRoles() {
		return roles;
	}

	/**
	 * Returns the salaryHistory.
	 * @return SalaryHistory
	 */
	public SalaryHistory getSalaryHistory() {
		return salaryHistory;
	}

	/**
	 * Sets the breaksInService.
	 * @param breaksInService The breaksInService to set
	 */
	public void setBreaksInService(Set breaksInService) {
		this.breaksInService = breaksInService;
	}

	/**
	 * Sets the company.
	 * @param company The company to set
	 */
	public void setCompany(Company company) {
		this.company = company;
	}

	/**
	 * Sets the companyBranch.
	 * @param companyBranch The companyBranch to set
	 */
	public void setCompanyBranch(CompanyBranch companyBranch) {
		this.companyBranch = companyBranch;
	}

	/**
	 * Sets the companyPayPoint.
	 * @param companyPayPoint The companyPayPoint to set
	 */
	public void setCompanyPayPoint(CompanyPayPoint companyPayPoint) {
		this.companyPayPoint = companyPayPoint;
	}

	/**
	 * Sets the costCentre.
	 * @param costCentre The costCentre to set
	 */
	public void setCostCentre(CostCentre costCentre) {
		this.costCentre = costCentre;
	}

	/**
	 * Sets the currentBreakInService.
	 * @param currentBreakInService The currentBreakInService to set
	 */
	public void setCurrentBreakInService(DateRange currentBreakInService) {
		this.currentBreakInService = currentBreakInService;
	}

	/**
	 * Sets the employeeCategory.
	 * @param employeeCategory The employeeCategory to set
	 */
	public void setEmployeeCategory(int employeeCategory) {
		this.employeeCategory = employeeCategory;
	}

	/**
	 * Sets the employeeID.
	 * @param employeeID The employeeID to set
	 */
	public void setEmployeeID(EmployeeID employeeID) {
		this.employeeID = employeeID;
	}

	/**
	 * Sets the officeEMailAddress.
	 * @param officeEMailAddress The officeEMailAddress to set
	 */
	public void setOfficeEMailAddress(EmailAddress officeEMailAddress) {
		this.officeEMailAddress = officeEMailAddress;
	}

	/**
	 * Sets the officeFaxNumber.
	 * @param officeFaxNumber The officeFaxNumber to set
	 */
	public void setOfficeFaxNumber(TelephoneNumber officeFaxNumber) {
		this.officeFaxNumber = officeFaxNumber;
	}

	/**
	 * Sets the officeNumber.
	 * @param officeNumber The officeNumber to set
	 */
	public void setOfficeNumber(TelephoneNumber officeNumber) {
		this.officeNumber = officeNumber;
	}

	/**
		 * Sets the roles.
		 * @param roles The roles to set
		 */
	public void addRole(Role role) {
		roles.add(role);
	}
	/**
	 * Sets the roles.
	 * @param roles The roles to set
	 */
	public void setRoles(Set roles) {
		this.roles = roles;
	}

	/**
	 * Sets the salaryHistory.
	 * @param salaryHistory The salaryHistory to set
	 */
	public void setSalaryHistory(SalaryHistory salaryHistory) {
		this.salaryHistory = salaryHistory;
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return employeeID.hashCode();
	}

	/**
	 * @see java.lang.Comparable#compareTo(Object)
	 */
	public int compareTo(Object o) {
		if (o instanceof Employee){
			return employeeID.compareTo(((Employee)o).getEmployeeID());
		}
		return this.getClass().getName().compareTo(o.getClass().getName());
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object o) {
		if (o instanceof Employee){
			return employeeID.equals(((Employee)o).getEmployeeID());
		}
		return false;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return employeeID.toString(); 
	}

}
