package hhenson.oodb.roles.employee;

import java.util.Set;

import hhenson.configuration.Configuration;
import hhenson.oodb.entity.Entity;
import hhenson.oodb.entity.company.Product;
import hhenson.oodb.roles.OodbContainedRoleImpl;

/**
 * A sales position/role/set of repsonsibilities that is attributed to an Employee role
 * and is associated with the employee for a period of time.
 * A sales role indicated the person has a sales responsibility and as such
 * will be associated with clients.
 * 
 * @author Howard Henson
 */
public class SalesRole extends OodbContainedRoleImpl {

	/**
	 * The set of clients are the Entities that the sales
	 * role is associated with. These include both people and companys.
	 */
	private Set clients;

	/**
	 * The products that a sale role is able or qualified to sell.
	 */
	private Set products;

	/**
	 * Provides a more detailed title to associate with this sales
	 * role. For example Scheme Consultant, which would imply an
	 * instance of a sales role with the stigma associated with the
	 * Scheme Consultant title.
	 */
	private String salesTitle;

	/**
	 * Constructor for SalesRole.
	 */
	public SalesRole() {
		super();
	}

	/**
	 * Returns the clients.
	 * @return Set
	 */
	public Set getClients() {
		return clients;
	}

	/**
	 * Returns the products.
	 * @return Set
	 */
	public Set getProducts() {
		return products;
	}

	/**
	 * Returns the salesTitle.
	 * @return String
	 */
	public String getSalesTitle() {
		return salesTitle;
	}

	/**
		 * Adds an entity to the client set
		 * @param entity The client/entity to set
		 */
	public void addClient(Entity entity) {
		clients.add(entity);
	}
	/**
	 * Sets the clients.
	 * @param clients The clients to set
	 */
	public void setClients(Set clients) {
		this.clients = clients;
	}

	/**
		 * Adds a product to the product set
		 * @param product The product to set
		 */
	public void addProduct(Product product) {
		products.add(product);
	}
	/**
	 * Sets the products.
	 * @param products The products to set
	 */
	public void setProducts(Set products) {
		this.products = products;
	}

	/**
	 * Sets the salesTitle.
	 * @param salesTitle The salesTitle to set
	 */
	public void setSalesTitle(String salesTitle) {
		this.salesTitle = salesTitle;
	}

	public static final String CFG_CLIENTS="Clients";
	public static final String CFG_PRODUCTS="Products";
	public static final String CFG_SALES_TITLE="SalesTitle";
	/**
	 * @see hhenson.configuration.Configurable#cfgLoad(Configuration)
	 */
	public void cfgLoad(Configuration config) {
		super.cfgLoad(config);
		clients=(Set)config.getValue(CFG_CLIENTS);
		products=(Set)config.getValue(CFG_PRODUCTS);
		salesTitle=(String)config.getValue(CFG_SALES_TITLE);
	}

	/**
	 * @see hhenson.configuration.Configurable#cfgStore()
	 */
	public Configuration cfgStore() {
		Configuration config=super.cfgStore();
		config.setValue(CFG_CLIENTS, clients);
		config.setValue(CFG_PRODUCTS, products);
		config.setValue(CFG_SALES_TITLE, salesTitle);
		return config;
		
	}


	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return super.hashCode()^clients.hashCode()^products.hashCode()^salesTitle.hashCode();
	}

}
