package hhenson.oodb.roles.users;

import java.util.Date;

import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;

import hhenson.core.InvalidIDException;
import hhenson.core.ObjectNotLocatedException;
import hhenson.oodb.core.OodbIDImpl;
import hhenson.roles.user.*;
import hhenson.roles.user.AuthenticationToken;
import hhenson.roles.user.UserID;
import hhenson.services.user.UserManagementService;

/**
 * The unique representation of a user within the system.
 * 
 * @author Howard Henson
 */
public class OodbUserIDImpl extends OodbIDImpl implements UserID {

	/**
	 * A twelve hour timeout on an authentication.
	 */
	private static final int TIMEOUT = 1000 * 60 * 60 * 12;

	private String sessionID = null;

	private long authenticatedTime = 0;

	private String userID;

	private transient UserManagementService userManagementService;

	/**
	 * Constructor for UserID.
	 */
	public OodbUserIDImpl() {
		super();
	}

	/**
	 * Constructs a new userID from a string value.
	 */
	public OodbUserIDImpl(String userID) {
		super();
		this.userID = userID;
	}

	/**
	 * @see java.lang.Comparable#compareTo(Object)
	 */
	public int compareTo(Object o) {
		if( o instanceof UserID ){
			String str = ((UserID)o).getUserName();
			return userID.compareTo(str);
		}
		return getClass().getName().compareTo(o.getClass().getName());
	}

	/**
	 * @see hhenson.oodb.core.ID#fetchObjectFromDatabase()
	 */
	protected Object fetchObjectFromDatabase()
		throws InvalidIDException, ObjectNotLocatedException {
		try {
			User user = userManagementService.getUser(userID);
			if( user == null ){
				throw new ObjectNotLocatedException( "User management service returned a null user!" );
			}
			return user;
		} catch (Exception e) {
			throw new ObjectNotLocatedException( "Failed to fetch: " + userID + " from user service", e );
		}
	}

	/**
	 * @see hhenson.roles.user.UserID#getUserName()
	 */
	public String getUserName() {
		return userID;
	}

	public boolean authenticateUser(
		AuthenticationToken token,
		String sessionID)
		throws InvalidIDException, ObjectNotLocatedException, UserLockedException {
		if (token != null) {
			User user = (User) getObject();
			if (user != null && user.authenticate(token)) {
				authenticatedTime = System.currentTimeMillis();
				this.sessionID = sessionID;
				return true;
			}
		}
		authenticatedTime = 0;
		sessionID = null;
		return false;
	}

	/**
	 * @see hhenson.roles.user.UserID#isAuthenticated()
	 */
	public boolean isAuthenticated() {
		if (authenticatedTime == 0) {
			return false;
		}
		if (System.currentTimeMillis() - authenticatedTime < TIMEOUT) {
			return true;
		}
		return false;
	}

	/**
	 * @see hhenson.roles.user.UserID#getSessionID()
	 */
	public String getSessionID() {
		return sessionID;
	}

	/**
	 * Set the user management service object that created this id object,
	 * this is probably not correct, but is is the easiest solution in the short term.
	 */
	public void setUserManagementService( UserManagementService _userManagementService){
		userManagementService = _userManagementService;
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object obj) {
		if( obj instanceof UserID ){
			return userID.equals(((UserID)obj).getUserName());
		}
		return false;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return userID;
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return userID.hashCode();
	}

}
