package hhenson.oodb.roles.users;

import goodsjpi.Metaobject;
import hhenson.configuration.Configuration;
import hhenson.core.ID;
import hhenson.oodb.roles.OodbRoleImpl;
import hhenson.roles.user.*;

import java.util.Date;

/**
 * A user is a role associated with a person that is used to 
 * identify an ability to make use of the applications
 * associated with the object in the database.
 * 
 * A user has: <br>
 * <ul><li><b>Profiles</b> - A profile contains information regarding what the
 * user has available to them in terms of services and the configurations
 * associated with the service.</li>
 * <li><b>Authentication Information</b> - Information that is utilised
 * in order to be able to authenticate the individual. An example
 * could be a password token.</li>
 * <li><b>UserID</b> - The user id to associated with this user role.</li>
 * </ul>
 * 
 * @author Howard Henson
 */
public class OodbUserImpl extends OodbRoleImpl implements User {

	private static final int INITIAL_LOCK_TIMEOUT = 5 * 1000;

	private UserID userID;

	/**
	 * The authentication token associated with this user.
	 * This is set as a single property untill it becomes
	 * useful to manage more than one authentication token.
	 */
	private AuthenticationToken token;

	/**
	 * The date and time when the users profile was last locked.
	 */
	private Date lockedDate;

	/**
	 * The amount of time to wait before allowing the user to
	 * try and gain access to the site after an incorrect password guess.
	 */
	private int lockTimeOut = INITIAL_LOCK_TIMEOUT;

	/**
	 * current number of consecutive failed authentication attempts.
	 */
	private int failedAuthenticationAttempts = 0;

	/**
	 * Constructor for User.
	 */
	public OodbUserImpl() {
		super();
	}

	/**
	 * The prefered mechanism of constructing a new userID.
	 */
	public OodbUserImpl(UserID userID) {
		super();
		this.userID = userID;
	}

	/**
	 * @see java.lang.Comparable#compareTo(Object)
	 */
	public int compareTo(Object o) {
		if (o instanceof User) {
			return userID.compareTo(((User) o).getId());
		}
		return this.getClass().getName().compareTo(o.getClass().getName());
	}

	/**
	 * @see hhenson.configuration.Configurable#cfgLoad(Configuration)
	 */
	public void cfgLoad(Configuration config) {
	}

	/**
	 * @see hhenson.configuration.Configurable#cfgStore()
	 */
	public Configuration cfgStore() {
		return null;
	}

	/**
	 * @see hhenson.oodb.core.Identifiable#getId()
	 */
	public ID getId() {
		return userID;
	}

	/**
	 * Determines if the user account is locked.
	 */
	public boolean isLocked() {
		if (lockedDate == null
			|| System.currentTimeMillis() - lockedDate.getTime() > lockTimeOut) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 * The logic to process when an authentication fails
	 */
	protected void failedAuthentication() {
		failedAuthenticationAttempts++;
		if (failedAuthenticationAttempts >= 3) {
			failedAuthenticationAttempts = 0;
			lockedDate = new Date();
			lockTimeOut *= 2;
		}
	}

	/**
	 * The logic to process when an authentication is successful
	 */
	protected void successfulAuthentication() {
		failedAuthenticationAttempts = 0;
		lockTimeOut = INITIAL_LOCK_TIMEOUT;
	}

	/**
	 * Attempts to authenticate the user using the given token. If the
	 * token does not correspond with the correct token the failed login
	 * count is incremented. After 3 failed attempts the User will stop
	 * trying to authenticate and will throw a UserLockedException irrespective of the
	 * value passed. The lock will reset itself after a timeout
	 * determined by the number of failed logon attempts. Each time the system locks
	 * itself the timeout is incremented by a power of two. After a successful authentication
	 * all the paramters are reset to their default values.
	 */
	public boolean authenticate(AuthenticationToken givenToken)
		throws UserLockedException {
		if (isLocked()) {
			throw new UserLockedException(
				"User: " + userID + " locked on date: " + lockedDate);
		}
		if (token.equals(givenToken)) {
			successfulAuthentication();
			return true;
		} else {
			failedAuthentication();
			return false;
		}
	}

	/**
	 * Returns the authentication token. This is probably most useful for debugging purposes.
	 */
	public AuthenticationToken getAuthenticationToken() {
		return token;
	}

	/**
	 * Sets the token.This should only be done by an authorised process.
	 * Need to put something along those lines in place!
	 * @param token The token to set
	 */
	public void setToken(AuthenticationToken token) {
		Metaobject.modify();
		this.token = token;
	}

}
