package hhenson.oodb.rules;

import goodsjpi.Persistent;
import hhenson.configuration.Configurable;
import hhenson.configuration.Configuration;
import hhenson.core.ID;
import hhenson.core.Identifiable;

/**
 * Interprets the input values and determines what needs
 * to be done in order to meet the requirements of the rule.
 * <p>
 * The rule takes the following main forms:<br>
 * <ul><li><b>Compliance rules</b> - determines whether the input meets
 * a compliance rule. The result of this is a percentage match (represented
 * as a number between 0 and 1).</li>
 * <li><b>Calculated rules</b> - calculates one or more properties based on
 * the inputs provided and returns the results of the calculations. An
 * example of this could be calculating the vat % on a transaction charge.</li>
 * <li><b>Derived rules</b> - infers a set of properties based on the input
 * data. An example could be determining a contribution category based on
 * the amount earned value of an employee.</li>
 * <li><b>Routing rule</b> - determines what rules need to be processed in
 * order to complete assesment of the input/s. The order of the rules should be
 * followed exactly as the rules may be order dependant. For example what
 * rules apply to processing a redemption.</li>
 * <li><b>Action rule</b> - Performs actions within the system, exteral to the rule,
 * that are controlled by the input and the policy implemented by the rule. An example
 * is perform a redemption, this would be an action rule that would result in the redemption
 * transaction being loaded for payment, stored in the database, and logged to
 * an audit trail. The action rule is also likely to make use of compliance rules
 * to determine exactly what actions to take.</li>
 * </ul>
 * </p>
 * <p>The result of a rule also contains termination information, in that if this
 * rule results in an end condition the rules execution engine should stop
 * processing rules and return the result.
 * </p>
 * <p>A rule can contain other rules that are used to evaluate this rule.
 * An example would be where the rule is to determine wherther or not
 * a transaction can be processed. To determine this a number of validation
 * rules may need to be applied, and the rule could stipulate conditions
 * such as rule1 must result in a 50% complience match. The rule
 * should contain all execution paths that could result in order to 
 * successfully complete. That is in the above example if rule1 is greater
 * that 50% then a positive route should exist as well as the negative should the
 * result be less than or equal to.
 * </p>
 * <p>A top level rules that are required to be identified and executed from the
 * exection engine must be uniquely identifiable. Rules contained within a rule may
 * be uniquely identifiable, but this not a requirement.
 * </p>
 * @author Howard Henson
 */
public abstract class Rule
	extends Persistent
	implements Configurable, Identifiable {

	/**
	 * Identifies the rule to be of type compliance rule.
	 */
	public static final int COMPLIANCE_RULE = 1;

	/**
	 * Identifies the rule as a calculated rule.
	 */
	public static final int CALCULATED_RULE = 2;

	/**
	 * Identifies the rule as a derived rule.
	 */
	public static final int DERIVED_RULE = 4;

	/**
	 * Identifies the rule as a routing rule.
	 */
	public static final int ROUTING_RULE = 8;

	/**
	 * Identifies the rule as an action rule.
	 */
	public static final int ACTION_RULE = 16;

	/**
	 * Defines the rule type as specified in the rule static definitions.
	 * Each implementation of a rule must set this value to one of the
	 * pre-defined values for a rule.
	 */
	private int ruleType;

	/**
	 * A short description of the rule.
	 * For example: Check if divorce agreement in place.
	 */
	private String ruleDescription;

	/**
	 * Constructor for Rule.
	 */
	public Rule() {
		super();
	}

	/**
	 * Returns the ruleType.
	 * @return int
	 */
	public int getRuleType() {
		return ruleType;
	}

	/**
	 * Sets the ruleType.
	 * @param ruleType The ruleType to set
	 */
	protected void setRuleType(int ruleType) {
		this.ruleType = ruleType;
	}

	/**
	 * Executes the rule against the parameters passed, returns the result of the rule
	 * execution.
	 * This is what should be overridden when implementing a new rule type.
	 */
	public abstract RuleResults execute(RuleParameters parameters);

	/**
	 * @see hhenson.configuration.Configurable#cfgLoad(Configuration)
	 */
	public static final String CFG_RULE_TYPE="RuleType";
	public static final String CFG_RULE_DESCRIPTION="RuleDescription";
	
	public void cfgLoad(Configuration config) {
		String type=(String)config.getValue(CFG_RULE_TYPE);
		if (type!=null){
			ruleType=Integer.parseInt(type);	
		}
		ruleDescription=(String)config.getValue(CFG_RULE_DESCRIPTION);
	}

	/**
	 * @see hhenson.configuration.Configurable#cfgStore()
	 */
	public Configuration cfgStore() {
		Configuration config=Configuration.newInstance();
		config.setValue(CFG_RULE_TYPE, Integer.toString(ruleType));
		config.setValue(CFG_RULE_DESCRIPTION, ruleDescription);
		return config;
	}

	/**
	 * Returns the ruleDescription.
	 * @return String
	 */
	public String getRuleDescription() {
		return ruleDescription;
	}

	/**
	 * Sets the ruleDescription.
	 * @param ruleDescription The ruleDescription to set
	 */
	public void setRuleDescription(String ruleDescription) {
		this.ruleDescription = ruleDescription;
	}

	/**
	 * @see hhenson.oodb.core.Identifiable#getId()
	 */
	public ID getId() {
		return null;
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return ruleType^ruleDescription.hashCode();
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object obj) {
		if (obj instanceof Rule){
			Rule rule=(Rule)obj;
			if (ruleType!=rule.getRuleType()){
				return false;	
			}
			if (!ruleDescription.equals(rule.getRuleDescription())){
				return false;
			}
			return true;
		}
		return false;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return super.toString();
	}

}
