package hhenson.oodb.services.ocache;

import goodsjpi.Database;
import goodslib.PersistentHashMap;
import hhenson.services.ocache.ObjectInitialContextFactory;
import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.NamingException;

import org.apache.avalon.excalibur.naming.DefaultNameParser;
import org.apache.avalon.excalibur.naming.DefaultNamespace;
import org.apache.avalon.framework.activity.Disposable;
import org.apache.avalon.framework.activity.Initializable;
import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.configuration.Reconfigurable;
import org.apache.avalon.framework.context.ContextException;
import org.apache.avalon.framework.context.Contextualizable;
import org.apache.avalon.framework.logger.LogEnabled;
import org.apache.avalon.framework.logger.Logger;
import org.apache.avalon.framework.thread.SingleThreaded;
import org.apache.avalon.phoenix.BlockContext;

/**
 * Obtains a reference to the initial context.
 * Obtains a reference to the database, then attempts
 * to fetch the initial context by looking up the root object.
 * 
 * Database configuration files should be stored in the /etc path
 * of the sar file.
 * 
 * @phoenix:block
 * @phoenix:service name="hhenson.services.ocache.ObjectInitialContextFactory"
 * @author Howard Henson
 */
public class OodbObjectInitialContextFactoryImpl
	implements
		ObjectInitialContextFactory,
		Configurable,
		Contextualizable,
		Reconfigurable,
		SingleThreaded,
		LogEnabled,
		Initializable,
		Disposable {

	private String dbName;
	private String pathToDb = "";
	private Database database;
	private Logger m_logger;
	private BlockContext m_context;

	/**
	 * Constructor for OodbObjectInitialContextFactoryImpl.
	 */
	public OodbObjectInitialContextFactoryImpl() {
		super();
	}

	/**
	 * @see javax.naming.spi.InitialContextFactory#getInitialContext(Hashtable)
	 */
	public Context getInitialContext(Hashtable environment)
		throws NamingException {
		final DefaultNameParser parser = new DefaultNameParser();
		final DefaultNamespace namespace = new DefaultNamespace(parser);
		PersistentHashMap map = null;
		Database db = getDatabase();
		if (db == null) {
			getLogger().fatalError("Did not get a reference to the database");
			return null;
		}
		Object obj = db.getRoot();
		if (obj != null ){
			if( obj instanceof PersistentHashMap) {
				map = (PersistentHashMap) obj;
			} else {
				//can't be a context root node
				getLogger().fatalError(
					"Dabase does not not have correct root object. Failed to construct context");
				return null;
			}
		} else {
			//No root object set, so we need to create the
			//context, then set the root to this context
			getLogger().info("No root object, constructing a new root context");
			map = new PersistentHashMap(40);
			db.setRoot(map);
		}
		return new OodbObjectCacheContextImpl(
			namespace,
			environment,
			null,
			map);
	}

	/**
	 * Looks up the database to use.
	 * This should make use of the context to get the database name to fetch.
	 */
	protected synchronized final Database getDatabase() {
		return database;
	}

	/**
	 * @see org.apache.avalon.framework.configuration.Configurable#configure(Configuration)
	 */
	public void configure(Configuration configuration)
		throws ConfigurationException {
		dbName = configuration.getValue();
		database = null;
	}

	/**
	 * @see org.apache.avalon.framework.configuration.Reconfigurable#reconfigure(Configuration)
	 */
	public void reconfigure(Configuration configuration)
		throws ConfigurationException {
		dbName = configuration.getValue();
		database = null;
	}

	/**
	 * @see org.apache.avalon.framework.logger.LogEnabled#enableLogging(Logger)
	 */
	public void enableLogging(Logger logger) {
		m_logger = logger;
	}

	/**
	 * returns this instances logger.
	 */
	protected final Logger getLogger() {
		return m_logger;
	}
	
	/**
	 * @see org.apache.avalon.framework.activity.Initializable#initialize()
	 */
	public void initialize() throws Exception {
		if (dbName == null) {
			getLogger().fatalError(
				"No database name set, can't construct database");
			throw new IllegalStateException("Database name not set");
		}
		if (database != null) {
			getLogger().error("Database already set, must have already been initialized");
			throw new IllegalStateException("Database already exists");
		}
		getLogger().info("Fetching new reference to database: " + pathToDb + dbName);
		database = Database.getDatabase(pathToDb + dbName);
	}

	/**
	 * @see org.apache.avalon.framework.activity.Disposable#dispose()
	 */
	public void dispose() {
		if( database == null ){
			getLogger().error("Database is not set and were trying to dispose");
			throw new IllegalStateException("Database does not exist");
		}
		database.close();
		database = null;
		dbName = null;
		pathToDb = null;
		m_context = null;
		m_logger = null;
	}

	/**
	 * @see org.apache.avalon.framework.context.Contextualizable#contextualize(Context)
	 */
	public void contextualize(
		org.apache.avalon.framework.context.Context context)
		throws ContextException {
		m_context = (BlockContext) context;
		pathToDb = m_context.getBaseDirectory() + "/etc/";
	}

}
