package hhenson.oodb.services.user;

import javax.naming.spi.InitialContextFactory;

import goodslib.PersistentHashMap;
import hhenson.core.InvalidIDException;
import hhenson.core.ObjectNotLocatedException;
import hhenson.oodb.roles.users.OodbAuthenticationTokenImpl;
import hhenson.oodb.roles.users.OodbUserIDImpl;
import hhenson.oodb.roles.users.OodbUserImpl;
import hhenson.roles.user.User;
import hhenson.roles.user.UserID;
import hhenson.roles.user.UserLockedException;
import hhenson.services.ocache.ObjectCacheContext;
import hhenson.services.ocache.ObjectInitialContextFactory;
import hhenson.services.user.UserManagementService;

import org.apache.avalon.framework.activity.Disposable;
import org.apache.avalon.framework.activity.Initializable;
import org.apache.avalon.framework.logger.LogEnabled;
import org.apache.avalon.framework.logger.Logger;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;

/**
 * Implements a user management service within an OODB.
 * 
 * @phoenix:block
 * @phoenix:service name="hhenson.services.user.UserManagementService"
 * @author Howard Henson
 */
public class OodbUserManagementServiceImpl
	implements UserManagementService, LogEnabled, Serviceable, Initializable, Disposable {

	private Logger logger;
	private ServiceManager serviceManager;
	private PersistentHashMap userMap;

	/**
	 * Constructor for OodbUserManagementServiceImpl.
	 */
	public OodbUserManagementServiceImpl() {
		super();
	}

	/**
	 * Sets the transient service manager propery on the outgoing ID.
	 * This may not be the best way to do this, but it should work in the short term.
	 */
	protected void setServiceToUserID(UserID userID){
		try{
			((OodbUserIDImpl)userID).setUserManagementService(this);
		} catch( Exception e ){
			logger.error("Failed to set service on UserID", e);
		}
	}
	
	/**
	 * @see hhenson.services.user.UserManagementService#createNewUser(String, String)
	 */
	public User createNewUser(String userName, String password) throws IllegalStateException {
		if( userMap.containsKey(userName) ){
			String msg = "User: " + userName + " already exists in the database";
			logger.error(msg );
			throw new IllegalStateException(msg);
		}
		User user = new OodbUserImpl(new OodbUserIDImpl(userName));
		user.setToken(new OodbAuthenticationTokenImpl(password));
		setServiceToUserID((UserID)user.getId());
		userMap.put(userName, user);
		return user;
	}

	/**
	 * @see hhenson.services.user.UserManagementService#deleteUser(UserID)
	 */
	public void deleteUser(UserID userID) throws IllegalStateException {
		deleteUser(userID.getUserName());
	}

	/**
	 * @see hhenson.services.user.UserManagementService#getUser(String)
	 */
	public User getUser(String userIDStr) {
		User user = (User) userMap.get(userIDStr);
		if( user == null ){
			logger.error("User: " + userIDStr + " does not exist");
		} else {
			setServiceToUserID((UserID)user.getId());
		}
		return user;
	}

	/**
	 * @see hhenson.services.user.UserManagementService#getUser(UserID)
	 */
	public User getUser(UserID userID) {
		return getUser(userID.getUserName());
	}

	/**
	 * @see hhenson.services.user.UserManagementService#logOff(UserID)
	 */
	public void logOff(UserID userID) {
		//lookup user session data and delete it
		try{
			//invalidate userID
			userID.authenticateUser(null, null);
		}catch(Exception e){
		}
	}

	/**
	 * @see hhenson.services.user.UserManagementService#logOn(String, String)
	 */
	public UserID logOn(String userIDStr, String password, String sessionID) {
		UserID userID = new OodbUserIDImpl(userIDStr);
		setServiceToUserID(userID);
		try{
			OodbUserImpl user = (OodbUserImpl) userID.getObject();
			if( user == null ){
				logger.error("Null user returned without throwing the correct exception");
			}
			logger.debug("User name: " + userIDStr);
			logger.debug("Password: " + user.getAuthenticationToken().toString());
			if( userID.authenticateUser(new OodbAuthenticationTokenImpl(password), sessionID) ){
				logger.info("User: " + userIDStr + " successfully authenticated");
				return userID;
			} else {
				logger.info("User: " + userIDStr + " failed to authenticated, returning null");
				return null;
			}
		} catch( InvalidIDException e ){
			logger.error("ID was found to be invalid whilst trying to login", e);
			return null;
		} catch( ObjectNotLocatedException e ){
			logger.error("User: " + userIDStr + " not found", e );
			return null;
		} catch( UserLockedException e ){
			logger.error("User: " + userIDStr + " attempting to log in and account is locked", e );
			return null;
		}
	}

	/**
	 * @see org.apache.avalon.framework.logger.LogEnabled#enableLogging(Logger)
	 */
	public void enableLogging(Logger logger) {
		this.logger = logger;
	}

	/**
	 * @phoenix:dependency name="hhenson.services.ocache.ObjectInitialContextFactory"
	 * @see org.apache.avalon.framework.service.Serviceable#service(ServiceManager)
	 */
	public void service(ServiceManager manager) throws ServiceException {
		serviceManager = manager;
	}
	
	private static final String USER_BINDING_POINT = "Users";

	/**
     * 
	 * @see org.apache.avalon.framework.activity.Initializable#initialize()
	 */
	public void initialize() throws Exception {
		InitialContextFactory ctxFactory =
			(InitialContextFactory) serviceManager.lookup(
				ObjectInitialContextFactory.ROLE);
		ObjectCacheContext ctx = (ObjectCacheContext) ctxFactory.getInitialContext(null);
		if( !ctx.isBound(USER_BINDING_POINT) ){
			logger.info("No user binding point found, add a new user collection");
			ctx.bind(USER_BINDING_POINT, new PersistentHashMap(500));
		}
		try{
			userMap = (PersistentHashMap) ctx.lookup(USER_BINDING_POINT);
		}catch( Exception e ){
			logger.fatalError("Could not get usermap binding, this is very bad", e);
			throw e;
		}
		serviceManager.release(ctxFactory);
	}

	/**
	 * @see org.apache.avalon.framework.activity.Disposable#dispose()
	 */
	public void dispose() {
		userMap = null;
		serviceManager = null;
		logger = null;
	}

	/**
	 * @see hhenson.services.user.UserManagementService#hasUser(String)
	 */
	public boolean hasUser(String userIDStr) {
		return userMap.containsKey(userIDStr);
	}

	/**
	 * @see hhenson.services.user.UserManagementService#hasUser(UserID)
	 */
	public boolean hasUser(UserID userId) {
		return hasUser(userId.getUserName());
	}

	/**
	 * @see hhenson.services.user.UserManagementService#deleteUser(String)
	 */
	public void deleteUser(String userIDStr) throws IllegalStateException {
		if( !userMap.containsKey(userIDStr) ){
			String msg = "User: " + userIDStr + " does not exists in the database";
			logger.error(msg );
			throw new IllegalStateException(msg);
		}
		userMap.remove(userIDStr);
	}

}
