package hhenson.oodb.util;

import goodsjpi.Persistent;
import hhenson.configuration.*;
import hhenson.core.*;
import hhenson.oodb.core.IDBound;
import hhenson.oodb.entity.CountryCodes;

/**
 * Represnets money in the system.
 * The money consists of information surrounding currency and
 * the magnitude of the value in terms of the base currency unit.
 * For example value = 1.5 implies R1 and 50 cents in ZAR.
 * 
 * @author Howard Henson
 */
public class Money
	extends Persistent
	implements Cloneable, Comparable, Configurable, Field {

	/**
	 * represents the value of the money in terms of the base unit.
	 * For example value = 1.5 implies R1 and 50 cents in ZAR.
	 */
	private double value;

	/**
	 * The base currency of the value. This
	 * is the index to the CountryCodes object
	 * that hold the information.
	 * The default is ZAR.
	 */
	private int currency = CountryCodes.ZA;

	/**
	 * Constructor for Money.
	 */
	public Money() {
		super();
	}

	/**
	 * Sets up a new money object from the values provided.
	 */
	public Money(int currency, double value) {
		super();
		setCurrency(currency);
		setValue(value);
	}

	/**
	 * Returns the currency.
	 * @return int
	 */
	public int getCurrency() {
		return currency;
	}

	/**
	 * Returns the value.
	 * @return double
	 */
	public double getValue() {
		return value;
	}

	/**
	 * Sets the currency.
	 * @param currency The currency to set
	 */
	protected void setCurrency(int currency) {
		this.currency = currency;
	}

	/**
	 * Sets the value.
	 * @param value The value to set
	 */
	protected void setValue(double value) {
		this.value = value;
	}

	/**
	 * Compare will only compare with objects of the same
	 * currency at this point in time.
	 * @see java.lang.Comparable#compareTo(Object)
	 */
	public int compareTo(Object o) {
		if (o instanceof Money) {
			Money money = (Money) o;
			if (currency == money.currency) {
				if (value == money.value) {
					return 0;
				}
				return (value < money.value)
					? IDBound.LESS_THAN
					: IDBound.GREATER_THAN;
			} else {
				return new Integer(currency).compareTo(
					new Integer(money.currency));
			}
		}
		return getClass().getName().compareTo(o.getClass().getName());
	}

	public static final String CFG_CURRENCY_CODE = "Currency";
	public static final String CFG_VALUE = "Value";
	/**
	 * @see hhenson.configuration.Configurable#cfgLoad(Configuration)
	 */
	// <Currency>ZAR</Currency>
	// <Value>3434.43</Value>
	public void cfgLoad(Configuration config) {
		String currencyStr = (String) config.getValue(CFG_CURRENCY_CODE);
		currency = 0;
		//This is not exactly optimal! we can contruct a lookup map
		//to make things better.
		if (currencyStr != null) {
			for (int c = 0; c < CountryCodes.COUNTRY_NAMES.length; c++) {
				if (currencyStr
					.equals(
						CountryCodes
							.COUNTRY_NAMES[c][CountryCodes
							.COUNTRY_CURRENCY_CODE])) {
					currency = c;
					c = CountryCodes.COUNTRY_NAMES.length;
				}
			}
		}
		String valueStr = (String) config.getValue(CFG_VALUE);
		value = 0.0;
		if (valueStr != null) {
			value = Double.parseDouble(valueStr);
		}
	}

	/**
	 * @see hhenson.configuration.Configurable#cfgStore()
	 */
	public Configuration cfgStore() {
		Configuration conf = Configuration.newInstance();
		conf.setValue(
			CFG_CURRENCY_CODE,
			CountryCodes
				.COUNTRY_NAMES[currency][CountryCodes
				.COUNTRY_CURRENCY_CODE]);
		conf.setValue(CFG_VALUE, Double.toString(value));
		return conf;
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object obj) {
		if (obj instanceof Money) {
			Money money = (Money) obj;
			if (currency == money.currency) {
				return (value == money.value);
			}
		}
		return false;
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return CountryCodes
			.COUNTRY_NAMES[currency][CountryCodes
			.COUNTRY_CURRENCY_SYMBOL]
			+ " "
			+ value;
	}

	/**
	 * Adds the contents of money to this money object.
	 * If the currency is not the same a currency mismatch is thrown.
	 * The reference to this object is returned. This facilitates
	 * perfoming multiple operations on the object. For example
	 * m1.add(m2).add(m3) etc.
	 */
	public Field add(Field field) throws FieldMismatchException {
		if (field instanceof Money) {
			Money money = (Money) field;
			if (currency != money.currency) {
				throw new CurrencyMismatchException(this, money);
			}
			value += money.value;
			return this;
		} else {
			throw new FieldMismatchException(
				field.getClass().getName()
					+ " not in field "
					+ this.getClass().getName());
		}
	}

	/**
	 * Subtracts the contents of money to this money object.
	 * If the currency is not the same a currency mismatch is thrown.
	 * The reference to this object is returned. This facilitates
	 * perfoming multiple operations on the object. For example
	 * m1.subtract(m2).subtract(m3) etc.
	 */
	public Field subtract(Field field) throws FieldMismatchException {
		if (field instanceof Money) {
			Money money = (Money) field;
			if (currency != money.currency) {
				throw new CurrencyMismatchException(this, money);
			}
			value -= money.value;
			return this;
		} else {
			throw new FieldMismatchException(
				field.getClass().getName()
					+ " not in field "
					+ this.getClass().getName());
		}
	}

	/**
	 * Multiply the contents of money to this money object.
	 * If the currency is not the same a currency mismatch is thrown.
	 * The reference to this object is returned. This facilitates
	 * perfoming multiple operations on the object. For example
	 * m1.multiply(m2).multiply(m3) etc.
	 */
	public Field multiply(Field field) throws FieldMismatchException {
		if (field instanceof Money) {
			Money money = (Money) field;
			if (currency != money.currency) {
				throw new CurrencyMismatchException(this, money);
			}
			value *= money.value;
			return this;
		} else {
			throw new FieldMismatchException(
				field.getClass().getName()
					+ " not in field "
					+ this.getClass().getName());
		}
	}

	/**
	 * Divides the contents of money to this money object.
	 * If the currency is not the same a currency mismatch is thrown.
	 * The reference to this object is returned. This facilitates
	 * perfoming multiple operations on the object. For example
	 * m1.divide(m2).divide(m3) etc.
	 */
	public Field divide(Field field) throws FieldMismatchException {
		if (field instanceof Money) {
			Money money = (Money) field;
			if (currency != money.currency) {
				throw new CurrencyMismatchException(this, money);
			}
			value /= money.value;
			return this;
		} else {
			throw new FieldMismatchException(
				field.getClass().getName()
					+ " not in field "
					+ this.getClass().getName());
		}
	}

	/**
	 * Adds the object with the scalar value in dValue.
	 * The reference to this object is returned. This facilitates
	 * perfoming multiple operations on the object. For example
	 * m1.add(m2).add(m3) etc.
	 */
	public Money add(double dValue) {
		value += dValue;
		return this;
	}

	/**
	 * Substract the object with the scalar value in dValue.
	 * The reference to this object is returned. This facilitates
	 * perfoming multiple operations on the object. For example
	 * m1.subtract(m2).subtract(m3) etc.
	 */
	public Money subtract(double dValue) {
		value -= dValue;
		return this;
	}

	/**
	 * Multiplies the object with the scalar value in dValue.
	 * The reference to this object is returned. This facilitates
	 * perfoming multiple operations on the object. For example
	 * m1.multiply(m2).multiply(m3) etc.
	 */
	public Money multiply(double dValue) {
		value *= dValue;
		return this;
	}

	/**
	 * Divides the object with the scalar value in dValue.
	 * The reference to this object is returned. This facilitates
	 * perfoming multiple operations on the object. For example
	 * m1.divide(m2).divide(m3) etc.
	 */
	public Money divide(double dValue) {
		value /= dValue;
		return this;
	}

	/**
	 * @see java.lang.Object#clone()
	 */
	public Object clone() throws CloneNotSupportedException {
		return new Money(currency, value);
	}

	/**
	 * @see hhenson.core.Field#addNew(Field)
	 */
	public Field addNew(Field field) throws FieldMismatchException {
		try {
			return ((Money) this.clone()).add(field);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * @see hhenson.core.Field#divideNew(Field)
	 */
	public Field divideNew(Field field) throws FieldMismatchException {
		try {
			return ((Money) this.clone()).divide(field);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * @see hhenson.core.Field#multiplyNew(Field)
	 */
	public Field multiplyNew(Field field) throws FieldMismatchException {
		try {
			return ((Money) this.clone()).multiply(field);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

	/**
	 * @see hhenson.core.Field#subtractNew(Field)
	 */
	public Field subtractNew(Field field) throws FieldMismatchException {
		try {
			return ((Money) this.clone()).subtract(field);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}
	}

}
