package hhenson.oodb.util;

import java.text.SimpleDateFormat;
import java.util.Date;

import goodsjpi.Persistent;
import hhenson.configuration.Configurable;
import hhenson.configuration.Configuration;
import hhenson.oodb.core.IDBound;
import hhenson.utils.DateRange;

/**
 * A date range contains properties for from date and to date.
 * The object also provides some useful methods such as hasEndDate, etc.
 * 
 * @author Howard Henson
 */
public class OodbDateRangeImpl 
	extends Persistent 
	implements DateRange {

	/**
	 * The lower end of the date range.
	 * If this property is not set, then it is deemed
	 * to always have been; that is the begining of time.
	 */
	private Date fromDate;

	/**
	 * The upper end of the range.
	 * If this is not set we deem that to imply
	 * that the is no current end date and could mean
	 * the end of time or undecided.
	 */
	private Date toDate;

	/**
	 * Constructor for DateRange.
	 */
	public OodbDateRangeImpl() {
		super();
	}

	/**
	 * Constructs a new range object with its associated dates.
	 */
	public OodbDateRangeImpl(Date fromDate, Date toDate) {
		this.fromDate = fromDate;
		this.toDate = toDate;
	}
	/**
	 * Compares to other date range objects.
	 * This compares firstly on the fromDate then on
	 * the to date. If the fromDates are different
	 * then the comparison is returned, if one of the from
	 * dates is null, then the null date is deemed to be less
	 * than the populated date.
	 * Otherwise the toDates are compared. If one of the to
	 * dates is null then the null date is considered to be
	 * the larger. If both are null then they are deemed to be
	 * the same. Otherwise the comparison of the date is performed. 
	 * 
	 * @see java.lang.Comparable#compareTo(Object)
	 */
	public int compareTo(Object o) {
		if (o instanceof OodbDateRangeImpl) {
			OodbDateRangeImpl dateRange = (OodbDateRangeImpl) o;
			if (fromDate != null && dateRange.fromDate != null) {
				int rVal = fromDate.compareTo(dateRange.fromDate);
				if (rVal != 0) {
					return rVal;
				}
			} else if (fromDate != null || dateRange.fromDate != null) {
				return (fromDate == null)
					? IDBound.LESS_THAN
					: IDBound.GREATER_THAN;
			}
			if (toDate != null && dateRange.toDate != null) {
				return toDate.compareTo(dateRange.toDate);
			} else if (toDate == null && dateRange.toDate == null) {
				return 0;
			} else {
				return (toDate == null)
					? IDBound.GREATER_THAN
					: IDBound.LESS_THAN;
			}
		}
		return getClass().getName().compareTo(o.getClass().getName());
	}

	/**
	 * Determines if a range is contained within this range.
	 * Containment is determined if fromDate&lt;=dateRange.fromDate
	 * and toDate&gt;=dateRange.toDate.
	 */
	public boolean containsRange(DateRange dateRange) {
		if (fromDate != null && dateRange.getFromDate() != null) {
			int rVal = fromDate.compareTo(dateRange.getFromDate());
			if (rVal == IDBound.GREATER_THAN) {
				return false;
			}
		} else if (fromDate != null || dateRange.getFromDate() != null) {
			if( dateRange.getFromDate() == null ){
				return false;
			}
		}
		if (toDate != null && dateRange.getToDate() != null) {
			if( toDate.compareTo(dateRange.getToDate()) == IDBound.LESS_THAN ){
				return false;
			}
		} else if (dateRange.getToDate() == null) {
			return false;
		}
		return true;
	}

	public static final String CFG_FROM_DATE = "FromDate";
	public static final String CFG_TO_DATE = "ToDate";

	/**
	 * The configuration of the data object is:
	 * &lt;FromDate format='format'&gt;date&lt;/FromDate&gt;<br>
	 * &lt;ToDate format='format'&gt;date&lt;ToDate&gt;<br>
	 * @see hhenson.configuration.Configurable#cfgLoad(Configuration)
	 */
	public void cfgLoad(Configuration config) {
		fromDate = (Date) config.getValue(CFG_FROM_DATE);
		toDate = (Date) config.getValue(CFG_TO_DATE);
	}

	/**
	 * @see hhenson.configuration.Configurable#cfgStore()
	 */
	public Configuration cfgStore() {
		Configuration conf = Configuration.newInstance();
		conf.setValue(CFG_FROM_DATE, fromDate);
		conf.setValue(CFG_TO_DATE, toDate);
		return conf;
	}

	/**
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(Object obj) {
		if (obj instanceof OodbDateRangeImpl) {
			OodbDateRangeImpl dateRange = (OodbDateRangeImpl) obj;
			if (fromDate != null && dateRange.fromDate != null) {
				if (!fromDate.equals(dateRange.fromDate)) {
					return false;
				}
			} else if (!(fromDate == null && dateRange.fromDate == null)) {
				return false;
			}
			if (toDate != null && dateRange.toDate != null) {
				return toDate.equals(dateRange.toDate);
			} else if (toDate == null && dateRange.toDate == null) {
				return true;
			} else {
				return false;
			}
		}
		return false;
	}

	/**
	 * Returns a string representation of this date range 
	 * in the form <b>yyyy/MM/dd to yyyy/MM/dd</b>
	 * If iether of the dates are open, (i.e. null) then
	 * a tilder is retuned in it's position.
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		SimpleDateFormat df = new SimpleDateFormat("yyyy/MM/dd");
		StringBuffer buff = new StringBuffer();
		if (fromDate == null) {
			buff.append("~");
		} else {
			buff.append(df.format(fromDate));
		}
		buff.append(" to ");
		if (toDate == null) {
			buff.append("~");
		} else {
			buff.append(df.format(toDate));
		}
		return buff.toString();
	}

	/**
	 * Returns the fromDate.
	 * @return Date
	 */
	public Date getFromDate() {
		return fromDate;
	}

	/**
	 * Returns the toDate.
	 * @return Date
	 */
	public Date getToDate() {
		return toDate;
	}

	/**
	 * Sets the fromDate.
	 * @param fromDate The fromDate to set
	 */
	public void setFromDate(Date fromDate) {
		this.fromDate = fromDate;
	}

	/**
	 * Sets the toDate.
	 * @param toDate The toDate to set
	 */
	public void setToDate(Date toDate) {
		this.toDate = toDate;
	}

}
