package hhenson.services.testpack;

import java.util.Iterator;
import java.util.LinkedList;

import org.apache.avalon.framework.activity.Disposable;
import org.apache.avalon.framework.activity.Startable;
import org.apache.avalon.framework.configuration.Configurable;
import org.apache.avalon.framework.configuration.Configuration;
import org.apache.avalon.framework.configuration.ConfigurationException;
import org.apache.avalon.framework.context.Context;
import org.apache.avalon.framework.context.ContextException;
import org.apache.avalon.framework.context.Contextualizable;
import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.avalon.phoenix.BlockContext;

/**
 * The main test pack application.
 * 
 * @phoenix:block
 * @phoenix:service name="hhenson.services.testpack.MainTestPack"
 * @author Howard Henson
 */
public class MainTestPackImpl
	extends AbstractLogEnabled
	implements Runnable, MainTestPack, Contextualizable, Startable, Configurable, Disposable, Serviceable {

	private Thread thread;
	private volatile boolean stopNow = false;
	private LinkedList tests = new LinkedList();
	private BlockContext m_context;
	private ServiceManager m_serviceManager;

	/**
	 * Constructor for MainTestPackImpl.
	 */
	public MainTestPackImpl() {
		super();
	}

	/**
	 * @see org.apache.avalon.framework.context.Contextualizable#contextualize(Context)
	 */
	public void contextualize(Context context) throws ContextException {
		m_context = (BlockContext) context;
	}

	/**
	 * @see org.apache.avalon.framework.activity.Startable#start()
	 */
	public void start() throws Exception {
		thread = new Thread(this);
		thread.start();
	}

	/**
	 * @see org.apache.avalon.framework.activity.Startable#stop()
	 */
	public void stop() throws Exception {
		if (thread.isAlive()) {
			getLogger().error("Shutting down tests before it is completed");
			synchronized (this) {
				stopNow = true;
			}
			thread.join();
		}
	}

	protected final String TEST_CASES = "Test";
	/**
	 * Loads the configuration which should take the form of:<br>
	 * <code>&lt;Test&gt;<br>
	 * &nbs;&nbs;[Service name for test case]<br>
	 * &lt;/Test&gt;<br>
	 * ...</code>
	 * @see org.apache.avalon.framework.configuration.Configurable#configure(Configuration)
	 */
	// <Test>
	//   ContextTest
	// </Test>
	public void configure(Configuration configuration)
		throws ConfigurationException {
		Configuration[] configs = configuration.getChildren(TEST_CASES);
		for (int i = 0; i < configs.length; i++) {
			Configuration config = configs[i];
			tests.add(config.getValue());
		}
	}

	/**
	 * Process through a list of test cases. Check to ensure that the 
	 * stop bit has not been set before continuing on to the next one.
	 *
	 * @see hhenson.services.testpack.TestPack#test()
	 */
	public boolean test() {
		boolean rVal = true;
		Iterator i = tests.iterator();
		while (i.hasNext()) {
			synchronized (this) {
				if (stopNow) {
					return false;
				}
			}
			String serviceName = (String) i.next();
			TestPack testCode = null;
			try{
				testCode = (TestPack) m_serviceManager.lookup(serviceName);
				if( !testCode.test() ){
					getLogger().error( "Test: " + serviceName + " failed" );
					rVal = false;
				}
				m_serviceManager.release(testCode);
			} catch( Exception e ){
				getLogger().error("Failed to execute test", e);
				rVal = false;
			} finally {
				if( testCode != null ){
					m_serviceManager.release(testCode);
					testCode = null;
				}
			}
		}
		return rVal;
	}

	/**
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		boolean result = test();
		if( result ){
			getLogger().info("The tests ran successfully");
		} else {
			getLogger().info("The tests were not successful");
		}
		m_context.requestShutdown();
	}

	/**
	 * @see org.apache.avalon.framework.activity.Disposable#dispose()
	 */
	public void dispose() {
		m_context = null;
		tests = null;
		thread = null;
	}

	/**
	 * @phoenix:dependency name="hhenson.services.testpack.ContextTester"
	 * @phoenix:dependency name="hhenson.services.testpack.UserTester"
	 * @see org.apache.avalon.framework.service.Serviceable#service(ServiceManager)
	 */
	public void service(ServiceManager manager) throws ServiceException {
		m_serviceManager = manager;
	}

}
