package hhenson.services.user;

import hhenson.core.ID;
import hhenson.roles.user.*;
import hhenson.roles.user.AuthenticationToken;
import hhenson.roles.user.User;

/**
 * Provides the interface for the user management service.
 * Holds login information and other session information associated
 * with the user.
 * 
 * These are the basic user management notions.
 * 
 * @author Howard Henson
 */
public interface UserManagementService {
	public String ROLE = UserManagementService.class.getName();
	
	/**
	 * Constructs a new user from a string based username and their string based
	 * password. Later it may be applicable to extend this offering to create
	 * from an authentication token, but the current utilisation will be string
	 * password based.
	 * @throws IllegalStateException when the user already exists in the database.
	 */
	public User createNewUser( String userName, String password ) throws IllegalStateException;
	
	/**
	 * Deletes the user from the collection of available users.
	 * @throws IllegalStateException if the user does not exist in the database.
	 */
	public void deleteUser( UserID userID ) throws IllegalStateException;
	
	/**
	 * Deletes the user from the collection of available users.
	 * @throws IllegalStateException if the user does not exist in the database.
	 */
	public void deleteUser( String userIDStr ) throws IllegalStateException;
	
	/**
	 * Returns the user for this ID.
	 * If you have the userID rather user the ID.getObject() method
	 * as this holds internal cache information that could
	 * make the lookup even faster that is otherwise possible.
	 * The user id will make use of this method if all else fails.
	 */
	public User getUser( UserID userID );
	
	/**
	 * Returns the user for the string provided.
	 * This should contstruct a valid userID from the
	 * string and then find based on userID.
	 */
	public User getUser( String userIDStr );
	
	/**
	 * Determines if the user associated with the user id string exists or not.
	 */
	public boolean hasUser( String userIDStr );
	
	/**
	 * Determines if the user associated with the user id string exists or not.
	 */
	public boolean hasUser( UserID userId );
	
	/**
	 * Performs a system logon, this will register the user as
	 * logged onto the system, the userID returned will contain
	 * the autheticated session information and will be stored
	 * for forced logoff.
	 * It is recommended that all authentication of users is managed through this
	 * interface as other state information can be persisted through this interface.
	 */
	public UserID logOn( String userIDStr, String password, String sessionID );
	
	/**
	 * Logs the user out of the system and cleans up any shared information.
	 */
	public void logOff( UserID userID );
}
