package hhenson.tools;

import java.io.File;
import java.io.IOException;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.Execute;
import org.apache.tools.ant.types.Commandline;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Reference;

/**
 * An ant task extension to allow for running the mop compiler.
 * 
 * This task requires that the goodshome property be set in the task
 * or that the javamop compiler be accessable via the PATH statement.
 * Below is an example usage:<br>
 * <br>
 * <code>
 * &lt;taskdef name="mop" classname="hhenson.tools.Mop"&gt;<br>
 * &nbsp;&nbsp;&lt;classpath/&gt; .. the path to this class <br>
 * &lt;/taskdef&gt;<br>
 * <br>
 * &lt;mop package="goodsjpi"<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;classes="classes.txt"<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;srcdir="./src"<br>
 * &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;goodshome="/usr/local/goods"
 * &gt;<br>
 * &nbsp;&nbsp;&lt;classpath refid="project.class.path"/&gt;<br>
 * &lt;/mop&gt;<br>
 * </code>
 * <br>
 * Where srcdir is required and must be a directory. The rest of the properties are not
 * required. The classpath property is the standard classpath property as defined by ant.
 * 
 * @author Howard Henson
 * @version 1.0
 */
public class Mop extends Task {

	private String m_package = "goodsjpi";
	private File m_classes;
	private File m_srcDir;
	private Path m_classPath;
	private File m_goodsHome;

	/**
	 * Constructor for Mop.
	 */
	public Mop() {
		super();
	}

	/**
	 * The package option advises the mop compiler what
	 * package to add to the import list of the class.
	 */
	public void setPackage(final String _package) {
		m_package = _package;
	}

	/**
	 * The classes option sets the <code>-classes</code> parameter to the goods
	 * mop compiler.
	 * 
	 * <i>From goods java documentation</i><br>
	 * <p>
	 * It is possible to use the existing Java class library without changing sources
	 * by putting the list of persistent capable classes in some file and using JavaMOP
	 * with the "-classes" option. The following rules should be observed carefully
	 * when using this option: <br>
	 * <ul>
	 * <li>A class should not have more than one instance array component.</li>
	 * <li>If the instance array component is modified (by storing array elements),
	 * the method Metaobject.modify() should be called explicitly to mark the object as modified.</li>
	 * <li>All classes, which objects can be referenced from some other persistent capable class,
	 * should also be persistent capable (so persistent capable classes should form closed sets
	 * in the sense of interclass references).</li>
	 * </ul>
	 * </p>
	 * <p>
	 * The JavaMOP option <code>-classes</code> allows you to specify a list of classes,
	 * which should be controlled by metaobjects. In this case exactly one class with a
	 * <code>metaobject</code> instance field should be provided to JavaMOP.
	 * This is refered to as the MOP root class. JavaMOP will transform the definition of
	 * the classes, specified in the list, modifying them to derived from the MOP root class.
	 * The option <code>-classes</code> should be followed by a file name.
	 * This file should contain a list of full class names, separated by white space characters
	 * (new line, space, tabulation...). The class name should be written in the same format as
	 * the Java import statement. For example: <code>goodslib.ArrayOfChar</code> or <code>goodslib.*"</oode>
	 * If the '*' symbol is placed at the end of a class name, then all classes from the specified package
	 * and nested packages are implicitly derived from the MOP root class.
	 * Classes specified in the <code>-classes</code> option list should not be archived,
	 * otherwise JavaMOP will not be able to preprocess them.
	 * </p>
	 * <i>Some notes:</i>
	 * <p>
	 * Although this functionality is really powerful, it is probably easier if you are writting you own code
	 * to place the <code>extends Persistent</code> in your classes and not make use of this aspect
	 * of the functionality.
	 * </p>
	 */
	public void setClasses(final File _classes) {
		if (!_classes.exists()) {
			throw new BuildException(
				"Classes file: " + _classes + " does not exist");
		}
		if (_classes.isDirectory()) {
			throw new BuildException(
				"Classes property: "
					+ _classes
					+ " should be a file, not a directory");
		}
		m_classes = _classes;
	}

	/**
	 * Sets the directory containing the source files to execute the mop compiler against.
	 * <i>From goods java documentation</i><br>
	 * <p>
	 * You can specify only the name of the root directory, which will be recursively traversed by JavaMOP.
	 * Only the files with *.class or *.CLASS extension will be preprocessed by JavaMOP while traversing the tree.
	 * </p>
	 * <p>
	 * If you want to create archives with persistent capable classes (ZIP or JAR), they should be build without compression
	 * (-0 flag for zip or jar) in order to let JavaMOP to read this files. JavaMOP currently is not able to unpack compressed
	 * ZIP files. JavaMOP will not patch files extracted from archive.
	 * Also JavaMOP adds special string to the preprocessed class file, so it will not patch it next time
	 * (in other words it is possible to apply JavaMOP several times to the same file - result will not be changed).
	 * </p>
	 * <p>
	 * In order for JavaMop to correctly process and insert the correct bytecode in the class files, 
	 * it is necessary to specify ALL class files which are persistent capable OR access persistent
	 * capable objects. That is, don't processes the files individually, this won't work.
	 * </p>
	 * <i>Some notes:</i>
	 * <p>
	 * Jars containing pre-processed persistent capable classes that are used by the project
	 * should be specified as a class path entry in the build script. Note however that the classes
	 * must not be compressed as the javaMop cannot read these files.
	 * </p>
	 * <p>
	 * Note that the implementation here is slightly different to the full usage possible in the 
	 * javamop. This is in the interests of simplicity and making this feel like the javac
	 * tag in the build script.
	 * </p>
	 */
	public void setSrcdir(final File _srcDir) {
		m_srcDir = _srcDir;
		if (!m_srcDir.exists()) {
			throw new BuildException(
				"Source Directory: " + m_srcDir + " does not exist.");
		}

		if (m_srcDir.isFile()) {
			throw new BuildException(
				"Source Directory: " + m_srcDir + " is a file.");
		}
	}

	/**
	 * Sets the classpath of libraries to include in the mop exercies.
	 * This is the list of all jars containing persistence capable classes that
	 * are referenced by the classes defined in src directory. Note the jar
	 * files must not be compressed!<br>
	 */
	public void addClasspath(final Path _classPath) {
		if (m_classPath == null) {
			m_classPath = _classPath;
		} else {
			m_classPath.append(_classPath);
		}
	}

	/**
	 * Sets the classpath of libraries to include in the mop exercies.
	 * This is the list of all jars containing persistence capable classes that
	 * are referenced by the classes defined in src directory. Note the jar
	 * files must not be compressed!<br>
	 */
	public void setClasspath(final Path _classPath) {
		if (m_classPath == null) {
			m_classPath = _classPath;
		} else {
			m_classPath.append(_classPath);
		}
	}

	/**
	 * Sets the classpath of libraries to include in the mop exercies.
	 * This is the list of all jars containing persistence capable classes that
	 * are referenced by the classes defined in src directory. Note the jar
	 * files must not be compressed!<br>
	 */
	public Path getClasspath() {
		return m_classPath;
	}

	/**
	 * Sets the classpath of libraries to include in the mop exercies.
	 * This is the list of all jars containing persistence capable classes that
	 * are referenced by the classes defined in src directory. Note the jar
	 * files must not be compressed!<br>
	 */
	public Path createClasspath() {
		if (m_classPath == null) {
			m_classPath = new Path(project);
		}
		return m_classPath.createPath();
	}

	/**
	 * Adds a reference to a classpath defined elsewhere.
	 */
	public void setClasspathRef(final Reference r) {
		createClasspath().setRefid(r);
	}

	/**
	 * The path where goods is installed on the system.
	 */
	public void setGoodshome(final File _goodsHome) {
		m_goodsHome = _goodsHome;
		if (!m_goodsHome.exists()) {
			throw new BuildException(
				"Goods Home Directory: " + m_goodsHome + " does not exist.");
		}

		if (m_goodsHome.isFile()) {
			throw new BuildException(
				"Goods Home Directory: " + m_goodsHome + " is a file.");
		}
	}

	/**
	 * @see org.apache.tools.ant.Task#execute()
	 */
	public void execute() throws BuildException {
		if (m_srcDir == null) {
			throw new BuildException("No source directory defined");
		}
		//get our hands on the javamop tool
		String cmdName;
		if (m_goodsHome == null) {
			cmdName = "javamop";
		} else {
			cmdName =
				m_goodsHome.getAbsolutePath()
					+ m_goodsHome.separator
					+ "bin"
					+ m_goodsHome.separator
					+ "javamop";
		}

		Commandline commandLine = new Commandline();
		commandLine.setExecutable(cmdName);

		commandLine.createArgument().setValue("-package");
		commandLine.createArgument().setValue(m_package);

		if (m_classes != null) {
			commandLine.createArgument().setValue("-classes");
			commandLine.createArgument().setFile(m_classes);
		}

		commandLine.createArgument().setFile(m_srcDir);

		String[] cPath = m_classPath.list();
		for (int i = 0; i < cPath.length; i++) {
			String string = cPath[i];
			commandLine.createArgument().setValue(string);
		}

		System.out.println(commandLine.toString());

		//execute the javamop
		try {
			Execute exe = new Execute();
			exe.setAntRun(project);
			exe.setWorkingDirectory(project.getBaseDir());
			exe.setCommandline(commandLine.getCommandline());
			exe.execute();
			if (exe.getExitValue() != 0) {
				throw new BuildException("execution of javamop failed");
			}
		} catch (IOException e) {
			throw new BuildException(
				"Error running javamop compiler",
				e,
				location);
		}
		
		System.out.println("done");

	}

}
